@@ -2,7 +2,7 @@
 #
 # A Debug UI for the Hatari, part of Python Gtk Hatari UI
 #
-# Copyright (C) 2008-2019 by Eero Tamminen
+# Copyright (C) 2008-2022 by Eero Tamminen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,8 +39,8 @@ def dialog_apply_cb(widget, dialog):
 class SaveDialog:
     def __init__(self, parent):
         table, self.dialog = create_table_dialog(parent, "Save from memory", 3, 2)
-        self.file = FselEntry(self.dialog)
-        table_add_widget_row(table, 0, 0, "File name:", self.file.get_container())
+        self.fsel = FselEntry(self.dialog, "Memory save file name:")
+        table_add_widget_row(table, 0, 0, "File name:", self.fsel.get_container())
         self.address = table_add_entry_row(table, 1, 0, "Save address:", 6)
         self.address.connect("activate", dialog_apply_cb, self.dialog)
         self.length = table_add_entry_row(table, 2, 0, "Number of bytes:", 6)
@@ -55,7 +55,7 @@ class SaveDialog:
         while 1:
             response = self.dialog.run()
             if response == Gtk.ResponseType.APPLY:
-                filename = self.file.get_filename()
+                filename = self.fsel.get_filename()
                 address_txt = self.address.get_text()
                 length_txt = self.length.get_text()
                 if filename and address_txt and length_txt:
@@ -84,11 +84,11 @@ class SaveDialog:
 
 class LoadDialog:
     def __init__(self, parent):
-        chooser = Gtk.FileChooserButton('Select a File')
+        chooser = Gtk.FileChooserButton(title="Select memory file to load:")
         chooser.set_local_only(True)  # Hatari cannot access URIs
         chooser.set_width_chars(12)
         table, self.dialog = create_table_dialog(parent, "Load to memory", 2, 2)
-        self.file = table_add_widget_row(table, 0, 0, "File name:", chooser)
+        self.fsel = table_add_widget_row(table, 0, 0, "File name:", chooser)
         self.address = table_add_entry_row(table, 1, 0, "Load address:", 6)
         self.address.connect("activate", dialog_apply_cb, self.dialog)
 
@@ -101,7 +101,7 @@ class LoadDialog:
         while 1:
             response = self.dialog.run()
             if response == Gtk.ResponseType.APPLY:
-                filename = self.file.get_filename()
+                filename = self.fsel.get_filename()
                 address_txt = self.address.get_text()
                 if filename and address_txt:
                     try:
@@ -121,20 +121,24 @@ class LoadDialog:
 class OptionsDialog:
     def __init__(self, parent):
         self.dialog = Gtk.Dialog("Debugger UI options", parent,
-            Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-            (Gtk.STOCK_APPLY,  Gtk.ResponseType.APPLY,
-             Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE))
+            modal=True, destroy_with_parent=True)
+        self.dialog.add_buttons(
+            Gtk.STOCK_APPLY,  Gtk.ResponseType.APPLY,
+            Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE
+        )
+        lines = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, 0, 50, 5)
+        lines.set_tooltip_text("Change view mode afterwards to get new (min size) into effect")
+        lines.set_digits(0)
+        self.lines = lines
 
-        self.lines = Gtk.Adjustment(0, 5, 50)
-        scale = Gtk.HScale(adjustment=self.lines)
-        scale.set_digits(0)
-
-        self.follow_pc = Gtk.CheckButton("On stop, set address to PC")
+        follow = Gtk.CheckButton(label="On stop, set address to PC")
+        follow.set_tooltip_text("Run-time option, not saved to Debugger config file")
+        self.follow_pc = follow
 
         vbox = self.dialog.vbox
         vbox.add(Gtk.Label(label="Memdump/disasm lines:"))
-        vbox.add(scale)
-        vbox.add(self.follow_pc)
+        vbox.add(lines)
+        vbox.add(follow)
         vbox.show_all()
 
     def run(self, lines, follow_pc):
@@ -155,6 +159,7 @@ class OptionsDialog:
 # constants for the other classes
 class Constants:
     # dump modes
+    NODUMP = 0
     DISASM = 1
     MEMDUMP = 2
     REGISTERS = 3
@@ -178,7 +183,7 @@ class MemoryAddress:
         # widgets
         self.entry, self.memory = self.create_widgets()
         # settings
-        self.dumpmode = Constants.REGISTERS
+        self.dumpmode = Constants.NODUMP
         self.follow_pc = True
         self.lines = 12
         # addresses
@@ -194,12 +199,12 @@ class MemoryAddress:
         self.last  = None
 
     def create_widgets(self):
-        entry = Gtk.Entry(max_length=6, width_chars=6)
+        entry = Gtk.Entry(max_length=6, width_chars=7)
         entry.connect("activate", self._entry_cb)
-        memory = Gtk.Label(halign=Gtk.Align.START, margin_start=8, margin_end=8, margin_top=8)
-        mono = Pango.FontDescription("monospace")
-        memory.modify_font(mono)
-        entry.modify_font(mono)
+        memory = Gtk.TextView()
+        memory.set_monospace(True)
+        memory.set_cursor_visible(False)
+        memory.set_editable(False)
         return (entry, memory)
 
     def _entry_cb(self, widget):
@@ -216,7 +221,7 @@ class MemoryAddress:
     def get(self):
         return self.first
 
-    def get_memory_label(self):
+    def get_memory_view(self):
         return self.memory
 
     def get_address_entry(self):
@@ -235,13 +240,16 @@ class MemoryAddress:
         self.lines = lines
 
     def set_dumpmode(self, mode):
+        if mode == self.dumpmode:
+            return
         self.dumpmode = mode
         self.dump()
 
     def dump(self, address = None, move_idx = 0):
         if self.dumpmode == Constants.REGISTERS:
             output = self._get_registers()
-            self.memory.set_label("".join(output))
+            text = self.memory.get_buffer()
+            text.set_text("".join(output))
             return
 
         if not address:
@@ -260,7 +268,8 @@ class MemoryAddress:
         else:
             print("ERROR: unknown dumpmode:", self.dumpmode)
             return
-        self.memory.set_label("".join(output))
+        text = self.memory.get_buffer()
+        text.set_text("".join(output))
         if move_idx:
             self.reset_entry()
 
@@ -299,6 +308,9 @@ class MemoryAddress:
         self.second = address + linewidth
         return output
 
+    def _get_line_addr(self, line, offset):
+        return int(line[offset:line.find(' ')], 16)
+
     def _get_disasm(self, address, move_idx):
         # TODO: uses brute force i.e. ask for more lines that user has
         # requested to be sure that the window is filled, assuming
@@ -332,10 +344,16 @@ class MemoryAddress:
                 output = output[-self.lines:]
             else:
                 output = output[:self.lines]
-        # with disasm need to re-get the addresses from the output
-        self.first  = int(output[0][1:output[0].find(":")], 16)
-        self.second = int(output[1][1:output[1].find(":")], 16)
-        self.last   = int(output[-1][1:output[-1].find(":")], 16)
+        # remove symbol lines from the output for address checking
+        addresses = [line for line in output if ' ' in line]
+        # address offset for UAE core and external disassembler output
+        offset = 0
+        if addresses[0][0] == '$':
+            offset = 1
+        # need to re-get the addresses from the disassembly output
+        self.first  = self._get_line_addr(addresses[0], offset)
+        self.second = self._get_line_addr(addresses[1], offset)
+        self.last   = self._get_line_addr(addresses[-1], offset)
         return output
 
     def _set_clamped(self, first, last):
@@ -355,8 +373,9 @@ class MemoryAddress:
 class HatariDebugUI:
 
     def __init__(self, hatariobj, do_destroy = False):
-        self.address = MemoryAddress(hatariobj)
         self.hatari = hatariobj
+        self.address = MemoryAddress(hatariobj)
+        self.address.set_dumpmode(Constants.REGISTERS)
         # set when needed/created
         self.dialog_load = None
         self.dialog_save = None
@@ -376,7 +395,7 @@ class HatariDebugUI:
         self.create_top_buttons(hbox1)
 
         # disasm/memory dump at the middle
-        addr = self.address.get_memory_label()
+        addr = self.address.get_memory_view()
 
         # buttons at bottom
         hbox2 = Gtk.HBox()
@@ -389,7 +408,7 @@ class HatariDebugUI:
         vbox.pack_start(hbox2, False, True, 0)
 
         # and the window for all of this
-        window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
+        window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
         window.set_events(Gdk.EventMask.KEY_RELEASE_MASK)
         window.connect("key_release_event", self.key_event_cb)
         if do_destroy:
@@ -411,16 +430,17 @@ class HatariDebugUI:
         buttons = (
             ("<<<", "Page_Up",  -Constants.MOVE_MAX),
             ("<<",  "Up",       -Constants.MOVE_MED),
-            ("<",   "Left",     -Constants.MOVE_MIN),
-            (">",   "Right",     Constants.MOVE_MIN),
+            ("<",   None,       -Constants.MOVE_MIN),
+            (">",   None,        Constants.MOVE_MIN),
             (">>",  "Down",      Constants.MOVE_MED),
             (">>>", "Page_Down", Constants.MOVE_MAX)
         )
         self.keys = {}
         for label, keyname, offset in buttons:
             button = create_button(label, self.set_address_offset, offset)
-            keyval = Gdk.keyval_from_name(keyname)
-            self.keys[keyval] =  offset
+            if keyname:
+                keyval = Gdk.keyval_from_name(keyname)
+                self.keys[keyval] =  offset
             box.add(button)
 
         # to middle of <<>> buttons
@@ -491,35 +511,35 @@ class HatariDebugUI:
     def options_cb(self, widget):
         if not self.dialog_options:
             self.dialog_options = OptionsDialog(self.window)
-        old_lines = self.config.get("[General]", "nLines")
-        old_follow_pc = self.config.get("[General]", "bFollowPC")
+        old_lines = self.config.get("[Debugger]", "nDisasmLines")
+        old_follow_pc = self.address.get_follow_pc()
         lines, follow_pc = self.dialog_options.run(old_lines, old_follow_pc)
         if lines != old_lines:
-            self.config.set("[General]", "nLines", lines)
+            self.config.set("[Debugger]", "nDisasmLines", lines)
             self.address.set_lines(lines)
         if follow_pc != old_follow_pc:
-            self.config.set("[General]", "bFollowPC", follow_pc)
             self.address.set_follow_pc(follow_pc)
+        self.address.dump()
 
     def load_options(self):
         # TODO: move config to MemoryAddress class?
         # (depends on how monitoring of addresses should work)
         lines = self.address.get_lines()
-        follow_pc = self.address.get_follow_pc()
-        miss_is_error = False # needed for adding windows
+        # ConfigStore does checks and type conversions based on names
+        # of Hatari config sections and keys, so this needs to use
+        # same names to avoid asserts, and it can't e.g. save
+        # follow_pc option value, which will keep as run-time one
         defaults = {
-            "[General]": {
-            	"nLines": lines,
-                "bFollowPC": follow_pc
+            "[Debugger]": {
+                "nDisasmLines": lines,
             }
         }
         userconfdir = ".hatari"
-        config = ConfigStore(userconfdir, defaults, miss_is_error)
-        configpath = config.get_filepath("debugui.cfg")
+        config = ConfigStore(userconfdir, defaults)
+        configpath = config.get_filepath(".debugui.cfg")
         config.load(configpath) # set defaults
         try:
-            self.address.set_lines(config.get("[General]", "nLines"))
-            self.address.set_follow_pc(config.get("[General]", "bFollowPC"))
+            self.address.set_lines(config.get("[Debugger]", "nDisasmLines"))
         except (KeyError, AttributeError):
             ErrorDialog(None).run("Debug UI configuration mismatch!\nTry again after removing: '%s'." % configpath)
         self.config = config
