@@ -25,8 +25,20 @@
 #define uae_log printf
 
 
+static inline void to_lower (TCHAR *s, int len) {
+	int i;
+	if (len < 0 )
+		len = _tcslen(s);
+	for (i = 0; i < len; i++) {
+		s[i] = tolower(s[i]);
+	}
+}
+
+
 static inline void to_upper (TCHAR *s, int len) {
 	int i;
+	if (len < 0 )
+		len = _tcslen(s);
 	for (i = 0; i < len; i++) {
 		s[i] = toupper(s[i]);
 	}
@@ -35,7 +47,7 @@ static inline void to_upper (TCHAR *s, int len) {
 
 static inline void my_trim (TCHAR *s)
 {
-	int len;
+	size_t len;
 	while (_tcslen (s) > 0 && _tcscspn (s, _T("\t \r\n")) == 0)
 		memmove (s, s + 1, (_tcslen (s + 1) + 1) * sizeof (TCHAR));
 	len = _tcslen (s);
