@@ -14,7 +14,7 @@
 #include "events.h"
 #include "newcpu.h"
 
-void do_cycles_slow (unsigned long cycles_to_add)
+void do_cycles_slow (uae_u32 cycles_to_add)
 {
 //fprintf ( stderr , "  do_cycles_slow add=%d curr=%d -> new=%d\n" , cycles_to_add , currcycle , currcycle+cycles_to_add );
 	currcycle += cycles_to_add;
