@@ -56,6 +56,7 @@ typedef void (*FPP_ABQS)(fpdata*, fpdata*, uae_u64*, uae_u8*);
 typedef void (*FPP_AB)(fpdata*, fpdata*);
 typedef void (*FPP_ABP)(fpdata*, fpdata*, int);
 typedef void (*FPP_A)(fpdata*);
+typedef void (*FPP_ABC)(fpdata*, fpdata*, fpdata*);
 
 typedef bool (*FPP_IS)(fpdata*);
 typedef void (*FPP_SET_MODE)(uae_u32);
@@ -156,6 +157,7 @@ extern FPP_AB fpp_cosh;
 extern FPP_ABP fpp_neg;
 extern FPP_AB fpp_acos;
 extern FPP_AB fpp_cos;
+extern FPP_ABC fpp_sincos;
 extern FPP_AB fpp_getexp;
 extern FPP_AB fpp_getman;
 extern FPP_ABP fpp_div;
