@@ -64,7 +64,7 @@
  #define DISABLE_I_SWAP
  #define DISABLE_I_EXG
  #define DISABLE_I_EXT
- #define DISABLE_I_MVEL
+ #define DISABLE_I_MVMEL
  #define DISABLE_I_MVMLE
  #define DISABLE_I_RTD
  #define DISABLE_I_LINK
@@ -829,7 +829,7 @@ static void genmov16(uae_u32 opcode, struct instr *curi)
 		comprintf("\tadd_l_ri(dstreg+8,16);\n");
 
 #ifdef UAE
-	comprintf("\tif (special_mem) {\n");
+	comprintf("\tif (special_mem || jit_n_addr_unsafe) {\n");
 	comprintf("\t\tint tmp=scratchie;\n");
 	comprintf("\tscratchie+=4;\n"
 		"\treadlong(src,tmp,scratchie);\n"
@@ -877,9 +877,9 @@ genmovemel(uae_u16 opcode)
 	genamode(table68k[opcode].dmode, "dstreg", table68k[opcode].size, "src", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_NO_INC);
 #ifdef UAE
 	if (table68k[opcode].size == sz_long)
-		comprintf("\tif (1 && !special_mem) {\n");
+		comprintf("\tif (1 && !special_mem && !jit_n_addr_unsafe) {\n");
 	else
-		comprintf("\tif (1 && !special_mem) {\n");
+		comprintf("\tif (1 && !special_mem && !jit_n_addr_unsafe) {\n");
 #endif
 
 	/* Fast but unsafe...  */
@@ -901,8 +901,8 @@ genmovemel(uae_u16 opcode)
 		break;
 	default: assert(0);
 	}
-	comprintf("\t\t}\n"
-		"\t}");
+	comprintf("\t\t}\n");
+	comprintf("\t}\n");
 	if (table68k[opcode].dmode == Aipi) {
 		comprintf("\t\t\tlea_l_brr(8+dstreg,srca,offset);\n");
 	}
@@ -923,13 +923,14 @@ genmovemel(uae_u16 opcode)
 		break;
 	case sz_word:
 		comprintf("\t\t\t\treadword(tmp,i,scratchie);\n"
+			"\t\t\t\tsign_extend_16_rr(i,i);\n"
 			"\t\t\t\tadd_l_ri(tmp,2);\n");
 		break;
 	default: assert(0);
 	}
 
-	comprintf("\t\t\t}\n"
-		"\t\t}\n");
+	comprintf("\t\t\t}\n");
+	comprintf("\t\t}\n");
 	if (table68k[opcode].dmode == Aipi) {
 		comprintf("\t\tmov_l_rr(8+dstreg,tmp);\n");
 	}
@@ -955,15 +956,18 @@ genmovemle(uae_u16 opcode)
 	   on her, but unfortunately, gfx mem isn't "real" mem, and thus that
 	   act of cleverness means that movmle must pay attention to special_mem,
 	   or Genetic Species is a rather boring-looking game ;-) */
-	if (table68k[opcode].size == sz_long)
-		comprintf("\tif (1 && !special_mem) {\n");
-	else
-		comprintf("\tif (1 && !special_mem) {\n");
+	if (table68k[opcode].dmode != Apdi) {
+		comprintf("\tif (1 && !special_mem && !jit_n_addr_unsafe) {\n");
+	} else {
+		// if Apdi and dstreg is included with mask: use indirect mode.
+		comprintf("\tif (1 && !special_mem && !jit_n_addr_unsafe && !(mask & (1 << (7 - dstreg)))) {\n");
+	}
 #endif
+
 	comprintf("\tget_n_addr(srca,native,scratchie);\n");
 
 	if (table68k[opcode].dmode != Apdi) {
-		comprintf("\tfor (i=0;i<16;i++) {\n"
+		comprintf("\tfor (i=0;i<16 && mask;i++) {\n"
 			"\t\tif ((mask>>i)&1) {\n");
 		switch (table68k[opcode].size) {
 		case sz_long:
@@ -981,7 +985,7 @@ genmovemle(uae_u16 opcode)
 		default: assert(0);
 		}
 	} else {  /* Pre-decrement */
-		comprintf("\tfor (i=0;i<16;i++) {\n"
+		comprintf("\tfor (i=0;i<16 && mask;i++) {\n"
 			"\t\tif ((mask>>i)&1) {\n");
 		switch (table68k[opcode].size) {
 		case sz_long:
@@ -1002,9 +1006,8 @@ genmovemle(uae_u16 opcode)
 		}
 	}
 
-
-	comprintf("\t\t}\n"
-		"\t}");
+	comprintf("\t\t}\n");
+	comprintf("\t}\n");
 	if (table68k[opcode].dmode == Apdi) {
 		comprintf("\t\t\tlea_l_brr(8+dstreg,srca,(uae_s32)offset);\n");
 	}
@@ -1013,7 +1016,7 @@ genmovemle(uae_u16 opcode)
 
 	if (table68k[opcode].dmode != Apdi) {
 		comprintf("\tmov_l_rr(tmp,srca);\n");
-		comprintf("\tfor (i=0;i<16;i++) {\n"
+		comprintf("\tfor (i=0;i<16 && mask;i++) {\n"
 			"\t\tif ((mask>>i)&1) {\n");
 		switch (table68k[opcode].size) {
 		case sz_long:
@@ -1027,7 +1030,7 @@ genmovemle(uae_u16 opcode)
 		default: assert(0);
 		}
 	} else {  /* Pre-decrement */
-		comprintf("\tfor (i=0;i<16;i++) {\n"
+		comprintf("\tfor (i=0;i<16 && mask;i++) {\n"
 			"\t\tif ((mask>>i)&1) {\n");
 		switch (table68k[opcode].size) {
 		case sz_long:
@@ -1042,9 +1045,8 @@ genmovemle(uae_u16 opcode)
 		}
 	}
 
-
-	comprintf("\t\t}\n"
-		"\t}");
+	comprintf("\t\t}\n");
+	comprintf("\t}\n");
 	if (table68k[opcode].dmode == Apdi) {
 		comprintf("\t\t\tmov_l_rr(8+dstreg,srca);\n");
 	}
@@ -1841,7 +1843,7 @@ gen_opcode(unsigned int opcode)
 		break;
 
 	case i_MVMEL:
-#ifdef DISABLE_I_MVEL
+#ifdef DISABLE_I_MVMEL
 		failure;
 #endif
 		genmovemel(opcode);
@@ -2320,41 +2322,55 @@ gen_opcode(unsigned int opcode)
 		if (curi->smode != immi) {
 			uses_cmov;
 			start_brace();
-			comprintf("\tint zero = scratchie++;\n");
+			comprintf("\tint cdata = scratchie++;\n");
 			comprintf("\tint tmpcnt = scratchie++;\n");
-			comprintf("\tint minus1 = scratchie++;\n");
-			comprintf("\tint cdata = minus1;\n");
-			comprintf("\tmov_l_rr(tmpcnt,cnt);\n");
-			comprintf("\tand_l_ri(tmpcnt,63);\n");
-			comprintf("\tmov_l_ri(zero, 0);\n");
-			comprintf("\tmov_l_ri(minus1, -1);\n");
+			comprintf("\tint setval = scratchie++;\n");
+			if (!noflags) {
+				comprintf("\tint odata = scratchie++;\n");
+			}
+			comprintf("\tmov_l_ri(cdata, 0);\n");
+			comprintf("\tmov_l_ri(setval, 0xffffffff);\n");
+			// if high bit = 0: setval = 0x00000000, else setval = 0xffffffff
+			comprintf("\ttest_l_ri(data, 0x%08x);\n", curi->size == sz_byte ? 0x80 : (curi->size == sz_word ? 0x8000 : 0x80000000));
+			comprintf("\tcmov_l_rr(setval, cdata, NATIVE_CC_EQ);\n");
+			comprintf("\tmov_l_rr(cdata, setval);\n");
+			if (!noflags) {
+				// setval -> odata
+				comprintf("\tmov_l_rr(odata, setval);\n");
+			}
+			comprintf("\tmov_l_rr(tmpcnt, cnt);\n");
+			comprintf("\tand_l_ri(tmpcnt, 63);\n");
+			if (!noflags) {
+				// shift == 0: tmpcnt (0) -> odata (C is always zero)
+				comprintf("\tcmov_l_rr(odata, tmpcnt, NATIVE_CC_EQ);\n");
+			}
+
 			switch (curi->size) {
 			case sz_byte:
-				comprintf("\ttest_b_rr(data,data);\n");
-				comprintf("\tcmov_l_rr(zero, minus1, NATIVE_CC_MI);\n");
-				comprintf("\ttest_l_ri(tmpcnt, 0x38);\n");
-				comprintf("\tmov_l_rr(cdata,data);\n");
-				comprintf("\tcmov_l_rr(cdata, zero, NATIVE_CC_NE);\n");
-				comprintf("\tshra_b_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_b_rr(data,cdata);\n");
+				comprintf("\tcmp_b_ri(tmpcnt, 0x08);\n");
+				// shift > 8: setval -> cdata
+				comprintf("\tcmov_l_rr(cdata, setval, NATIVE_CC_HI);\n");
+				// shift <= 8: data -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshra_b_rr(cdata, tmpcnt);\n");			
 				break;
 			case sz_word:
-				comprintf("\ttest_w_rr(data,data);\n");
-				comprintf("\tcmov_l_rr(zero, minus1, NATIVE_CC_MI);\n");
-				comprintf("\ttest_l_ri(tmpcnt, 0x30);\n");
-				comprintf("\tmov_l_rr(cdata,data);\n");
-				comprintf("\tcmov_l_rr(cdata, zero, NATIVE_CC_NE);\n");
-				comprintf("\tshra_w_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_w_rr(data,cdata);\n");
+				comprintf("\tcmp_b_ri(tmpcnt, 0x10);\n");
+				// shift > 16: setval -> cdata
+				comprintf("\tcmov_l_rr(cdata, setval, NATIVE_CC_HI);\n");
+				// shift <= 16: data -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshra_w_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_long:
-				comprintf("\ttest_l_rr(data,data);\n");
-				comprintf("\tcmov_l_rr(zero, minus1, NATIVE_CC_MI);\n");
-				comprintf("\ttest_l_ri(tmpcnt, 0x20);\n");
-				comprintf("\tmov_l_rr(cdata,data);\n");
-				comprintf("\tcmov_l_rr(cdata, zero, NATIVE_CC_NE);\n");
-				comprintf("\tshra_l_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_l_rr(data,cdata);\n");
+				comprintf("\tcmp_b_ri(tmpcnt, 0x20);\n");
+				// shift > 32: setval -> cdata
+				comprintf("\tcmov_l_rr(cdata, setval, NATIVE_CC_HI);\n");
+				// shift == 32? 0 -> cdata (x86 masks count by 31, 680x0 uses mask 63)
+				comprintf("\tcmov_l_rr(data, setval, NATIVE_CC_EQ);\n");
+				// shift <= 32: data -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshra_l_rr(cdata, tmpcnt);\n");
 				break;
 			default: assert(0);
 			}
@@ -2371,12 +2387,22 @@ gen_opcode(unsigned int opcode)
 		if (!noflags) {
 			comprintf("\tlive_flags();\n");
 			comprintf("\tend_needflags();\n");
-			if (curi->smode != immi)
-				comprintf("\tsetcc_for_cntzero(tmpcnt, data, %d);\n", curi->size == sz_byte ? 1 : curi->size == sz_word ? 2 : 4);
-			else
+			if (curi->smode != immi) {
+				comprintf("\tsetcc_for_cntzero(tmpcnt, cdata, odata, 0, %d, 0);\n", curi->size);
+			} else {
 				comprintf("\tduplicate_carry();\n");
+			}
 			comprintf("if (!(needed_flags & FLAG_CZNV)) dont_care_flags();\n");
 		}
+		if (curi->smode != immi) {
+			switch (curi->size) {
+			case sz_byte: comprintf("\tmov_b_rr(data, cdata);\n"); break;
+			case sz_word: comprintf("\tmov_w_rr(data, cdata);\n"); break;
+			case sz_long: comprintf("\tmov_l_rr(data, cdata);\n"); break;
+			default: assert(0);
+			}
+		}
+
 		genastore("data", curi->dmode, "dstreg", curi->size, "data");
 		break;
 
@@ -2414,28 +2440,51 @@ gen_opcode(unsigned int opcode)
 			uses_cmov;
 			start_brace();
 			comprintf("\tint cdata = scratchie++;\n");
-			comprintf("\tint tmpcnt=scratchie++;\n");
-			comprintf("\tmov_l_rr(tmpcnt,cnt);\n");
-			comprintf("\tand_l_ri(tmpcnt,63);\n");
+			comprintf("\tint tmpcnt = scratchie++;\n");
+			if (!noflags) {
+				comprintf("\tint odata = scratchie++;\n");
+				comprintf("\tmov_l_rr(odata, data);\n");
+			}
 			comprintf("\tmov_l_ri(cdata, 0);\n");
+			comprintf("\tmov_l_rr(tmpcnt, cnt);\n");
+			comprintf("\tand_l_ri(tmpcnt, 63);\n");
+			if (!noflags) {
+				// shift == 0? cdata (0) -> odata (C is always zero)
+				comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_EQ);\n");
+			}
+
 			switch (curi->size) {
 			case sz_byte:
-				comprintf("\ttest_l_ri(tmpcnt, 0x38);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshll_b_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_b_rr(data, cdata);\n");
+				// shift > 8? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x08);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift <= 8? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshll_b_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_word:
-				comprintf("\ttest_l_ri(tmpcnt, 0x30);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshll_w_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_w_rr(data, cdata);\n");
+				// shift > 16? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x10);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift <= 16? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshll_w_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_long:
-				comprintf("\ttest_l_ri(tmpcnt, 0x20);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshll_l_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_l_rr(data, cdata);\n");
+				// shift > 32? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x20);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift == 32? 0 -> cdata (x86 masks count by 31, 680x0 uses mask 63)
+				comprintf("\tcmov_l_rr(data, cdata, NATIVE_CC_EQ);\n");
+				// shift <= 32? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshll_l_rr(cdata, tmpcnt);\n");
 				break;
 			default: assert(0);
 			}
@@ -2452,12 +2501,22 @@ gen_opcode(unsigned int opcode)
 		if (!noflags) {
 			comprintf("\tlive_flags();\n");
 			comprintf("\tend_needflags();\n");
-			if (curi->smode != immi)
-				comprintf("\tsetcc_for_cntzero(tmpcnt, data, %d);\n", curi->size == sz_byte ? 1 : curi->size == sz_word ? 2 : 4);
-			else
+			if (curi->smode != immi) {
+				comprintf("\tsetcc_for_cntzero(tmpcnt, cdata, odata, 0, %d, 1);\n", curi->size);
+			} else {
+				comprintf("\tclear_overflow();\n");
 				comprintf("\tduplicate_carry();\n");
+			}
 			comprintf("if (!(needed_flags & FLAG_CZNV)) dont_care_flags();\n");
 		}
+		if (curi->smode != immi) {
+			switch (curi->size) {
+			case sz_byte: comprintf("\tmov_b_rr(data, cdata);\n"); break;
+			case sz_word: comprintf("\tmov_w_rr(data, cdata);\n"); break;
+			case sz_long: comprintf("\tmov_l_rr(data, cdata);\n"); break;
+			default: assert(0);
+			}
+		}
 		genastore("data", curi->dmode, "dstreg", curi->size, "data");
 		break;
 
@@ -2486,28 +2545,51 @@ gen_opcode(unsigned int opcode)
 			uses_cmov;
 			start_brace();
 			comprintf("\tint cdata = scratchie++;\n");
-			comprintf("\tint tmpcnt=scratchie++;\n");
-			comprintf("\tmov_l_rr(tmpcnt,cnt);\n");
-			comprintf("\tand_l_ri(tmpcnt,63);\n");
+			comprintf("\tint tmpcnt = scratchie++;\n");
+			if (!noflags) {
+				comprintf("\tint odata = scratchie++;\n");
+				comprintf("\tmov_l_rr(odata, data);\n");
+			}
 			comprintf("\tmov_l_ri(cdata, 0);\n");
+			comprintf("\tmov_l_rr(tmpcnt, cnt);\n");
+			comprintf("\tand_l_ri(tmpcnt, 63);\n");
+			if (!noflags) {
+				// shift == 0? cdata (0) -> odata (C is always zero)
+				comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_EQ);\n");
+			}
+			
 			switch (curi->size) {
 			case sz_byte:
-				comprintf("\ttest_l_ri(tmpcnt, 0x38);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshrl_b_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_b_rr(data, cdata);\n");
+				// shift > 8? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x08);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift <= 8? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshrl_b_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_word:
-				comprintf("\ttest_l_ri(tmpcnt, 0x30);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshrl_w_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_w_rr(data, cdata);\n");
+				// shift > 16? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x10);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift <= 16? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshrl_w_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_long:
-				comprintf("\ttest_l_ri(tmpcnt, 0x20);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
+				// shift > 32? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x20);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift == 32? 0 -> cdata (x86 masks count by 31, 680x0 uses mask 63)
+				comprintf("\tcmov_l_rr(data, cdata, NATIVE_CC_EQ);\n");
+				// shift <= 32? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
 				comprintf("\tshrl_l_rr(cdata, tmpcnt);\n");
-				comprintf("\tmov_l_rr(data, cdata);\n");
 				break;
 			default: assert(0);
 			}
@@ -2524,12 +2606,21 @@ gen_opcode(unsigned int opcode)
 		if (!noflags) {
 			comprintf("\tlive_flags();\n");
 			comprintf("\tend_needflags();\n");
-			if (curi->smode != immi)
-				comprintf("\tsetcc_for_cntzero(tmpcnt, data, %d);\n", curi->size == sz_byte ? 1 : curi->size == sz_word ? 2 : 4);
-			else
+			if (curi->smode != immi) {
+				comprintf("\tsetcc_for_cntzero(tmpcnt, cdata, odata, %d, %d, 1);\n", curi->size == sz_byte ? 7 : curi->size == sz_word ? 15 : 31, curi->size);
+			} else {
 				comprintf("\tduplicate_carry();\n");
+			}
 			comprintf("if (!(needed_flags & FLAG_CZNV)) dont_care_flags();\n");
 		}
+		if (curi->smode != immi) {
+			switch (curi->size) {
+			case sz_byte: comprintf("\tmov_b_rr(data, cdata);\n"); break;
+			case sz_word: comprintf("\tmov_w_rr(data, cdata);\n"); break;
+			case sz_long: comprintf("\tmov_l_rr(data, cdata);\n"); break;
+			default: assert(0);
+			}
+		}
 		genastore("data", curi->dmode, "dstreg", curi->size, "data");
 		break;
 
@@ -2559,27 +2650,50 @@ gen_opcode(unsigned int opcode)
 			start_brace();
 			comprintf("\tint cdata = scratchie++;\n");
 			comprintf("\tint tmpcnt = scratchie++;\n");
-			comprintf("\tmov_l_rr(tmpcnt,cnt);\n");
-			comprintf("\tand_l_ri(tmpcnt,63);\n");
+			if (!noflags) {
+				comprintf("\tint odata = scratchie++;\n");
+				comprintf("\tmov_l_rr(odata, data);\n");
+			}
 			comprintf("\tmov_l_ri(cdata, 0);\n");
+			comprintf("\tmov_l_rr(tmpcnt, cnt);\n");
+			comprintf("\tand_l_ri(tmpcnt, 63);\n");
+			if (!noflags) {
+				// shift == 0? cdata (0) -> odata (C is always zero)
+				comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_EQ);\n");
+			}
+
 			switch (curi->size) {
 			case sz_byte:
-				comprintf("\ttest_l_ri(tmpcnt, 0x38);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshll_b_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_b_rr(data, cdata);\n");
+				// shift > 8? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x08);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift <= 8? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshll_b_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_word:
-				comprintf("\ttest_l_ri(tmpcnt, 0x30);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshll_w_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_w_rr(data, cdata);\n");
+				// shift > 16? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x10);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift <= 16? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshll_w_rr(cdata, tmpcnt);\n");
 				break;
 			case sz_long:
-				comprintf("\ttest_l_ri(tmpcnt, 0x20);\n");
-				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_EQ);\n");
-				comprintf("\tshll_l_rr(cdata,tmpcnt);\n");
-				comprintf("\tmov_l_rr(data, cdata);\n");
+				// shift > 32? 0 -> odata (C is always zero)
+				comprintf("\tcmp_b_ri(tmpcnt, 0x20);\n");
+				if (!noflags) {
+					comprintf("\tcmov_l_rr(odata, cdata, NATIVE_CC_HI);\n");
+				}
+				// shift == 32? 0 -> cdata (x86 masks count by 31, 680x0 uses mask 63)
+				comprintf("\tcmov_l_rr(data, cdata, NATIVE_CC_EQ);\n");
+				// shift <= 32? cdata -> cdata ("normal" shift)
+				comprintf("\tcmov_l_rr(cdata, data, NATIVE_CC_LS);\n");
+				comprintf("\tshll_l_rr(cdata, tmpcnt);\n");
 				break;
 			default: assert(0);
 			}
@@ -2596,12 +2710,22 @@ gen_opcode(unsigned int opcode)
 		if (!noflags) {
 			comprintf("\tlive_flags();\n");
 			comprintf("\tend_needflags();\n");
-			if (curi->smode != immi)
-				comprintf("\tsetcc_for_cntzero(tmpcnt, data, %d);\n", curi->size == sz_byte ? 1 : curi->size == sz_word ? 2 : 4);
-			else
+			if (curi->smode != immi) {
+				comprintf("\tsetcc_for_cntzero(tmpcnt, cdata, odata, 0, %d, 1);\n", curi->size);
+			} else {
+				comprintf("\tclear_overflow();\n");
 				comprintf("\tduplicate_carry();\n");
+			}
 			comprintf("if (!(needed_flags & FLAG_CZNV)) dont_care_flags();\n");
 		}
+		if (curi->smode != immi) {
+			switch (curi->size) {
+			case sz_byte: comprintf("\tmov_b_rr(data, cdata);\n"); break;
+			case sz_word: comprintf("\tmov_w_rr(data, cdata);\n"); break;
+			case sz_long: comprintf("\tmov_l_rr(data, cdata);\n"); break;
+			default: assert(0);
+			}
+		}
 		genastore("data", curi->dmode, "dstreg", curi->size, "data");
 		break;
 
@@ -2631,6 +2755,17 @@ gen_opcode(unsigned int opcode)
 
 		if (!noflags) {
 			comprintf("\tstart_needflags();\n");
+
+			comprintf("\tint cdata = scratchie++;\n");
+			comprintf("\tint zero = scratchie++;\n");
+			comprintf("\tint tmpcnt = scratchie++;\n");
+			comprintf("\tmov_l_rr(cdata,data);\n");
+			comprintf("\tmov_l_rr(tmpcnt,cnt);\n");
+			comprintf("\tmov_l_ri(zero,0);\n");
+			// if shift count is zero: C is always cleared
+			comprintf("\tand_l_ri(tmpcnt,63);\n");
+			comprintf("\tcmov_l_rr(cdata,zero,NATIVE_CC_EQ);\n");
+
 			/*
 			 * x86 ROL instruction does not set ZF/SF, so we need extra checks here
 			 */
@@ -2641,7 +2776,7 @@ gen_opcode(unsigned int opcode)
 			case sz_long: comprintf("\ttest_l_rr(data,data);\n"); break;
 			}
 			comprintf("}\n");
-			comprintf("\tbt_l_ri(data,0x00);\n"); /* Set C */
+			comprintf("\tbt_l_ri(cdata,0x00);\n"); /* Set C */
 			comprintf("\tlive_flags();\n");
 			comprintf("\tend_needflags();\n");
 		}
@@ -2674,6 +2809,16 @@ gen_opcode(unsigned int opcode)
 
 		if (!noflags) {
 			comprintf("\tstart_needflags();\n");
+
+			comprintf("\tint cdata = scratchie++;\n");
+			comprintf("\tint zero = scratchie++;\n");
+			comprintf("\tint tmpcnt = scratchie++;\n");
+			comprintf("\tmov_l_rr(cdata,data);\n");
+			comprintf("\tmov_l_rr(tmpcnt,cnt);\n");
+			comprintf("\tmov_l_ri(zero,0);\n");
+			// if shift count is zero: C is always cleared
+			comprintf("\tand_l_ri(tmpcnt,63);\n");
+			comprintf("\tcmov_l_rr(cdata,zero,NATIVE_CC_EQ);\n");
 			/*
 			 * x86 ROR instruction does not set ZF/SF, so we need extra checks here
 			 */
@@ -2685,9 +2830,9 @@ gen_opcode(unsigned int opcode)
 			}
 			comprintf("}\n");
 			switch (curi->size) {
-			case sz_byte: comprintf("\tbt_l_ri(data,0x07);\n"); break;
-			case sz_word: comprintf("\tbt_l_ri(data,0x0f);\n"); break;
-			case sz_long: comprintf("\tbt_l_ri(data,0x1f);\n"); break;
+			case sz_byte: comprintf("\tbt_l_ri(cdata,0x07);\n"); break;
+			case sz_word: comprintf("\tbt_l_ri(cdata,0x0f);\n"); break;
+			case sz_long: comprintf("\tbt_l_ri(cdata,0x1f);\n"); break;
 			}
 			comprintf("\tlive_flags();\n");
 			comprintf("\tend_needflags();\n");
