@@ -44,6 +44,7 @@ const char DebugInfo_fileid[] = "Hatari debuginfo.c";
 #include "vdi.h"
 #include "video.h"
 #include "videl.h"
+#include "vme.h"
 #include "xbios.h"
 #include "newcpu.h"
 #include "68kDisass.h"
@@ -300,10 +301,6 @@ static void DebugInfo_PrintOSHeader(FILE *fp, Uint32 sysbase)
 	Uint32 gemblock, basepage;
 	Uint16 osversion, datespec, osconf, langbits;
 	const char *lang;
-	static const char langs[][3] = {
-		"us", "de", "fr", "uk", "es", "it", "se", "ch" /* fr */, "ch" /* de */,
-		"tr", "fi", "no", "dk", "sa", "nl", "cs", "hu"
-	};
 
 	/* first more technical info */
 
@@ -351,14 +348,9 @@ static void DebugInfo_PrintOSHeader(FILE *fp, Uint32 sysbase)
 
 	osconf = STMemory_ReadWord(sysbase+0x1C);
 	langbits = osconf >> 1;
-	if (langbits == 127) {
-		lang = "all";
-	} else if (langbits < ARRAY_SIZE(langs)) {
-		lang = langs[langbits];
-	} else {
-		lang = "unknown";
-	}
-	fprintf(fp, "OS config    : %s, %s (0x%x)\n", lang, osconf&1 ? "PAL":"NTSC", osconf);
+	lang = TOS_LanguageName(langbits);
+	fprintf(fp, "OS config    : %s (0x%x), %s (%d)\n",
+		osconf&1 ? "PAL":"NTSC", osconf, lang, langbits);
 	fprintf(fp, "Phystop      : %d KB\n", (STMemory_ReadLong(OS_PHYSTOP) + 511) / 1024);
 }
 
@@ -676,13 +668,14 @@ static const struct {
 	{ true, "dspmemdump",DebugInfo_DspMemDump, DebugInfo_DspMemArgs, "Dump DSP memory from given <space> <address>" },
 	{ true, "dspregs",   DebugInfo_DspRegister,NULL, "Show DSP register contents" },
 #endif
-	{ false, "dta",      DebugInfo_DTA,       NULL, "Show current [or given] DTA information" },
+	{ false, "dta",      DebugInfo_DTA,        NULL, "Show current [or given] DTA information" },
 	{ true, "file",      DebugInfo_FileParse, DebugInfo_FileArgs, "Parse commands from given debugger input <file>" },
 	{ false,"gemdos",    GemDOS_Info,          NULL, "Show GEMDOS HDD emu information (with <value>, show opcodes)" },
 	{ true, "history",   History_Show,         NULL, "Show history of last <count> instructions" },
 	{ false,"ikbd",      IKBD_Info,            NULL, "Show IKBD (SCI) register contents" },
 	{ true, "memdump",   DebugInfo_CpuMemDump, NULL, "Dump CPU memory from given <address>" },
 	{ false,"mfp",       MFP_Info,             NULL, "Show MFP register contents" },
+	{ false,"mmu",       M68000_MMU_Info,      NULL, "Show MMU register contents" },
 	{ false,"nvram",     NvRam_Info,           NULL, "Show (TT/Falcon) NVRAM contents" },
 	{ false,"osheader",  DebugInfo_OSHeader,   NULL, "Show TOS OS header contents" },
 	{ true, "regaddr",   DebugInfo_RegAddr, DebugInfo_RegAddrArgs, "Show <disasm|memdump> from CPU/DSP address pointed by <register>" },
@@ -692,6 +685,7 @@ static const struct {
 	{ false,"vdi",       VDI_Info,             NULL, "Show VDI vector contents (with <value>, show opcodes)" },
 	{ false,"videl",     Videl_Info,           NULL, "Show Falcon Videl register contents" },
 	{ false,"video",     Video_Info,           NULL, "Show Video information" },
+	{ false,"vme",       VME_Info,             NULL, "Show VME/SCU register information" },
 	{ false,"xbios",     XBios_Info,           NULL, "Show XBIOS opcodes" },
 	{ false,"ym",        PSG_Info,             NULL, "Show YM-2149 register contents" },
 };
