@@ -165,7 +165,7 @@ static bool DebugCpu_ShowAddressInfo(Uint32 addr, FILE *fp)
  */
 int DebugCpu_DisAsm(int nArgc, char *psArgs[])
 {
-	Uint32 disasm_upper = 0;
+	Uint32 prev_addr, disasm_upper = 0, pc = M68000_GetPC();
 	int shown, lines = INT_MAX;
 	uaecptr nextpc;
 
@@ -188,7 +188,7 @@ int DebugCpu_DisAsm(int nArgc, char *psArgs[])
 	{
 		/* continue */
 		if(!disasm_addr)
-			disasm_addr = M68000_GetPC();
+			disasm_addr = pc;
 	}
 
 	/* limit is topmost address or instruction count */
@@ -199,8 +199,21 @@ int DebugCpu_DisAsm(int nArgc, char *psArgs[])
 	}
 
 	/* output a range */
+	prev_addr = disasm_addr;
 	for (shown = 0; shown < lines && disasm_addr < disasm_upper; shown++)
 	{
+		if (prev_addr < pc && disasm_addr > pc)
+		{
+			fputs("ERROR, disassembly misaligned with PC address, correcting\n", debugOutput);
+			disasm_addr = pc;
+			shown++;
+		}
+		if (disasm_addr == pc)
+		{
+			fputs("(PC)\n", debugOutput);
+			shown++;
+		}
+		prev_addr = disasm_addr;
 		if (DebugCpu_ShowAddressInfo(disasm_addr, debugOutput))
 			shown++;
 		Disasm(debugOutput, (uaecptr)disasm_addr, &nextpc, 1);
@@ -348,11 +361,7 @@ int DebugCpu_Register(int nArgc, char *psArgs[])
 		int idx;
 
 		/* use the UAE function instead */
-#ifdef WINUAE_FOR_HATARI
 		m68k_dumpstate_file(debugOutput, &nextpc, 0xffffffff);
-#else
-		m68k_dumpstate(debugOutput, &nextpc);
-#endif
 		fflush(debugOutput);
 		if (!bFakeRegsUsed)
 			return DEBUGGER_CMDDONE;
@@ -714,7 +723,7 @@ static int DebugCpu_Continue(int nArgc, char *psArgv[])
 static int DebugCpu_Step(int nArgc, char *psArgv[])
 {
 	nCpuSteps = 1;
-	return DEBUGGER_END;
+	return DEBUGGER_ENDCONT;
 }
 
 
@@ -731,16 +740,32 @@ static char *DebugCpu_MatchNext(const char *text, int state)
 	return DebugUI_MatchHelper(ntypes, ARRAY_SIZE(ntypes), text, state);
 }
 
+/**
+ * Variable + debugger variable function for tracking
+ * subroutine call depth for "next" breakpoint
+ */
+static int CpuCallDepth;
+Uint32 DebugCpu_CallDepth(void)
+{
+	return CpuCallDepth;
+}
+/* Depth tracking can start anywhere i.e. it can go below initial
+ * value.  Start from large enough value that it should never goes
+ * negative, as then DebugCpu_CallDepth() return value would wrap
+ */
+#define CALL_START_DEPTH 10000
+
 /**
  * Command: Step CPU, but proceed through subroutines
  * Does this by temporary conditional breakpoint
  */
 static int DebugCpu_Next(int nArgc, char *psArgv[])
 {
-	char command[40];
+	char command[80];
 	if (nArgc > 1)
 	{
 		int optype;
+		bool depthcheck = false;
 		if(strcmp(psArgv[1], "branch") == 0)
 			optype = CALL_BRANCH;
 		else if(strcmp(psArgv[1], "exception") == 0)
@@ -750,7 +775,10 @@ static int DebugCpu_Next(int nArgc, char *psArgv[])
 		else if(strcmp(psArgv[1], "subcall") == 0)
 			optype = CALL_SUBROUTINE;
 		else if (strcmp(psArgv[1], "subreturn") == 0)
+		{
 			optype = CALL_SUBRETURN;
+			depthcheck = true;
+		}
 		else if (strcmp(psArgv[1], "return") == 0)
 			optype = CALL_SUBRETURN | CALL_EXCRETURN;
 		else
@@ -758,7 +786,18 @@ static int DebugCpu_Next(int nArgc, char *psArgv[])
 			fprintf(stderr, "Unrecognized opcode type given!\n");
 			return DEBUGGER_CMDDONE;
 		}
-		sprintf(command, "CpuOpcodeType & $%x > 0 :once :quiet\n", optype);
+		/* CpuOpCodeType increases call depth on subroutine calls,
+		 * and decreases depth on return from them, so it must be
+		 * first check to get called on every relevant instruction.
+		 */
+		if (depthcheck)
+		{
+			CpuCallDepth = CALL_START_DEPTH;
+			sprintf(command, "CpuOpcodeType & $%x > 0  &&  CpuCallDepth < $%x  :once :quiet\n",
+				optype, CALL_START_DEPTH);
+		}
+		else
+			sprintf(command, "CpuOpcodeType & $%x > 0 :once :quiet\n", optype);
 	}
 	else
 	{
@@ -782,14 +821,14 @@ static int DebugCpu_Next(int nArgc, char *psArgv[])
 		else
 		{
 			nCpuSteps = 1;
-			return DEBUGGER_END;
+			return DEBUGGER_ENDCONT;
 		}
 	}
 	/* use breakpoint, not steps */
 	if (BreakCond_Command(command, false))
 	{
 		nCpuSteps = 0;
-		return DEBUGGER_END;
+		return DEBUGGER_ENDCONT;
 	}
 	return DEBUGGER_CMDDONE;
 }
@@ -805,16 +844,21 @@ Uint32 DebugCpu_OpcodeType(void)
 	if (opcode == 0x4e74 ||			/* RTD */
 	    opcode == 0x4e75 ||			/* RTS */
 	    opcode == 0x4e77)			/* RTR */
+	{
+		CpuCallDepth--;
 		return CALL_SUBRETURN;
-
+	}
 	if (opcode == 0x4e73)			/* RTE */
+	{
 		return CALL_EXCRETURN;
-
+	}
 	/* NOTE: BSR needs to be matched before BRA/BCC! */
 	if ((opcode & 0xff00) == 0x6100 ||	/* BSR */
 	    (opcode & 0xffc0) == 0x4e80)	/* JSR */
+	{
+		CpuCallDepth++;
 		return CALL_SUBROUTINE;
-
+	}
 	/* TODO: ftrapcc, chk2? */
 	if (opcode == 0x4e72 ||			/* STOP */
 	    opcode == 0x4afc ||			/* ILLEGAL */
@@ -822,8 +866,9 @@ Uint32 DebugCpu_OpcodeType(void)
 	    (opcode & 0xfff0) == 0x4e40 ||	/* TRAP */
 	    (opcode & 0xf1c0) == 0x4180 ||	/* CHK */
 	    (opcode & 0xfff8) == 0x4848)	/* BKPT */
+	{
 		return CALL_EXCEPTION;
-
+	}
 	/* TODO: fbcc, fdbcc */
 	if ((opcode & 0xf000) == 0x6000 ||	/* BRA / BCC */
 	    (opcode & 0xffc0) == 0x4ec0 ||	/* JMP */
@@ -860,11 +905,7 @@ void DebugCpu_Check(void)
 	if (LOG_TRACE_LEVEL(TRACE_CPU_REGS))
 	{
 		uaecptr nextpc;
-#ifdef WINUAE_FOR_HATARI
 		m68k_dumpstate_file(TraceFile, &nextpc, 0xffffffff);
-#else
-		m68k_dumpstate(TraceFile, &nextpc);
-#endif
 	}
 	if (nCpuActiveCBs)
 	{
@@ -888,7 +929,7 @@ void DebugCpu_Check(void)
 	{
 		History_AddCpu();
 	}
-	if (ConOutDevice != CONOUT_DEVICE_NONE)
+	if (ConOutDevices)
 	{
 		Console_Check();
 	}
@@ -906,7 +947,7 @@ void DebugCpu_SetDebugging(void)
 
 	if (nCpuActiveCBs || nCpuSteps || bCpuProfiling || History_TrackCpu()
 	    || LOG_TRACE_LEVEL((TRACE_CPU_DISASM|TRACE_CPU_SYMBOLS|TRACE_CPU_REGS))
-	    || ConOutDevice != CONOUT_DEVICE_NONE)
+	    || ConOutDevices)
 	{
 		M68000_SetDebugger(true);
 		nCpuInstructions = 0;
@@ -934,8 +975,8 @@ static const dbgcommand_t cpucommands[] =
 	  "disasm", "d",
 	  "disassemble from PC, or given address",
 	  "[<start address>[-<end address>]]\n"
-	  "\tIf no address is given, this command disassembles from the last\n"
-	  "\tposition or from current PC if no last position is available.",
+	  "\tWhen no address is given, disassemble from the last disasm\n"
+	  "\taddress, or from current PC when debugger is (re-)entered.",
 	  false },
 	{ DebugCpu_Profile, Profile_Match,
 	  "profile", "",
@@ -946,8 +987,8 @@ static const dbgcommand_t cpucommands[] =
 	  "cpureg", "r",
 	  "dump register values or set register to value",
 	  "[REG=value]\n"
-	  "\tSet CPU register to value or dumps all register if no parameter\n"
-	  "\thas been specified.",
+	  "\tSet CPU register to given value, or dump all registers\n"
+	  "\twhen no parameter is given.",
 	  true },
 	{ DebugCpu_MemDump, Symbols_MatchCpuDataAddress,
 	  "memdump", "m",
@@ -969,13 +1010,13 @@ static const dbgcommand_t cpucommands[] =
 	{ DebugCpu_LoadBin, NULL,
 	  "loadbin", "l",
 	  "load a file into memory",
-	  "filename address\n"
+	  "<filename> <address>\n"
 	  "\tLoad the file <filename> into memory starting at <address>.",
 	  false },
 	{ DebugCpu_SaveBin, NULL,
 	  "savebin", "",
 	  "save memory to a file",
-	  "filename address length\n"
+	  "<filename> <address> <length>\n"
 	  "\tSave the memory block at <address> with given <length> to\n"
 	  "\tthe file <filename>.",
 	  false },
@@ -988,7 +1029,7 @@ static const dbgcommand_t cpucommands[] =
 	  "step", "s",
 	  "single-step CPU",
 	  "\n"
-	  "\tExecute next CPU instruction (equals 'c 1')",
+	  "\tExecute next CPU instruction (like 'c 1', but repeats on Enter).",
 	  false },
 	{ DebugCpu_Next, DebugCpu_MatchNext,
 	  "next", "n",
@@ -997,7 +1038,7 @@ static const dbgcommand_t cpucommands[] =
 	  "\tSame as 'step' command if there are no subroutine calls.\n"
           "\tWhen there are, those calls are treated as one instruction.\n"
 	  "\tIf argument is given, continues until instruction of given\n"
-	  "\ttype is encountered.",
+	  "\ttype is encountered.  Repeats on Enter.",
 	  false },
 	{ DebugCpu_Continue, NULL,
 	  "cont", "c",
