@@ -556,10 +556,10 @@ int Profile_AllocCallinfo(callinfo_t *callinfo, int count, const char *name)
 		/* alloc & clear new data */
 		callinfo->site = calloc(count, sizeof(callee_t));
 		if (callinfo->site) {
-			printf("Allocated %s profile callsite buffer for %d symbols.\n", name, count);
+			fprintf(stderr, "Allocated %s profile callsite buffer for %d symbols.\n", name, count);
 			callinfo->prev_pc = callinfo->return_pc = PC_UNDEFINED;
 		} else {
-			fprintf(stderr, "ERROR: callesite buffer alloc failed!\n");
+			fprintf(stderr, "ERROR: callsite buffer alloc failed!\n");
 			callinfo->sites = 0;
 		}
 	}
@@ -659,7 +659,8 @@ static bool Profile_Save(const char *fname, bool bForDsp)
 {
 	FILE *out;
 	Uint32 freq;
-	const char *proc, *core;
+	const char *proc;
+
 	if (!(out = fopen(fname, "w"))) {
 		fprintf(stderr, "ERROR: opening '%s' for writing failed!\n", fname);
 		perror(NULL);
@@ -672,12 +673,8 @@ static bool Profile_Save(const char *fname, bool bForDsp)
 		freq = MachineClocks.CPU_Freq_Emul;
 		proc = "CPU";
 	}
-#if ENABLE_WINUAE_CPU
-	core = "WinUAE";
-#else
-	core = "OldUAE";
-#endif
-	fprintf(out, "Hatari %s profile (%s, %s CPU core)\n", proc, PROG_NAME, core);
+
+	fprintf(out, "Hatari %s profile (%s)\n", proc, PROG_NAME);
 	fprintf(out, "Cycles/second:\t%u\n", freq);
 	if (bForDsp) {
 		Profile_DspSave(out);
@@ -779,9 +776,9 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 			lower = *disasm_addr;
 		}
 		if (bForDsp) {
-			*disasm_addr = Profile_DspShowAddresses(lower, upper, stdout, PAGING_ENABLED);
+			*disasm_addr = Profile_DspShowAddresses(lower, upper, stderr, PAGING_ENABLED);
 		} else {
-			*disasm_addr = Profile_CpuShowAddresses(lower, upper, stdout, PAGING_ENABLED);
+			*disasm_addr = Profile_CpuShowAddresses(lower, upper, stderr, PAGING_ENABLED);
 		}
 		return DEBUGGER_CMDCONT;
 
@@ -837,9 +834,9 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 		}
 	} else if (strcmp(psArgs[1], "callers") == 0) {
 		if (bForDsp) {
-			Profile_DspShowCallers(stdout);
+			Profile_DspShowCallers(stderr);
 		} else {
-			Profile_CpuShowCallers(stdout);
+			Profile_CpuShowCallers(stderr);
 		}
 	} else if (strcmp(psArgs[1], "stack") == 0) {
 		Profile_ShowStack(bForDsp);
