@@ -96,13 +96,11 @@ static struct {
 	Uint32 loop_end;      /* address of last loop end */
 	Uint32 loop_count;    /* how many times it was looped */
 	Uint32 disasm_addr;   /* 'addresses' command start address */
-#if ENABLE_WINUAE_CPU
 	Uint32 i_prefetched;  /* instructions that don't incur prefetch hit/miss */
 	Uint32 i_hit_counts[MAX_I_HITS];    /* I-cache hit counts */
 	Uint32 d_hit_counts[MAX_D_HITS];    /* D-cache hit counts */
 	Uint32 i_miss_counts[MAX_I_MISSES]; /* I-cache miss counts */
 	Uint32 d_miss_counts[MAX_D_MISSES]; /* D-cache miss counts */
-#endif
 	bool processed;	      /* true when data is already processed */
 	bool enabled;         /* true when profiling enabled */
 } cpu_profile;
@@ -160,10 +158,8 @@ static inline Uint32 address2index(Uint32 pc)
 			/* and after TOS as it's higher */
 			pc += TosSize;
 		}
-#if ENABLE_WINUAE_CPU
 	} else if (TTmemory && pc >= TTRAM_START && pc < TTRAM_START + 1024*(unsigned)ConfigureParams.Memory.TTRamSize_KB) {
 		pc += STRamEnd + TosSize + CART_SIZE - TTRAM_START;
-#endif
 	} else {
 		if (++cpu_warnings.address <= MAX_SHOW_COUNT) {
 			fprintf(stderr, "WARNING: 'invalid' CPU PC profile instruction address 0x%x!\n", pc);
@@ -387,7 +383,6 @@ void Profile_CpuShowStats(void)
 	show_cpu_warnings();
 }
 
-#if ENABLE_WINUAE_CPU
 /**
  * show percentage histogram of given array items
  */
@@ -437,11 +432,6 @@ void Profile_CpuShowCaches(void)
 	show_histogram("Data cache misses per instruction",
 		       ARRAY_SIZE(cpu_profile.d_miss_counts), cpu_profile.d_miss_counts);
 }
-#else
-void Profile_CpuShowCaches(void) {
-	fprintf(stderr, "Cache information is recorded only with WinUAE CPU.\n");
-}
-#endif
 
 /**
  * Show CPU instructions which execution was profiled, in the address order,
@@ -510,9 +500,11 @@ Uint32 Profile_CpuShowAddresses(Uint32 lower, Uint32 upper, FILE *out, paging_t
 		addrs++;
 	}
 	if (idx < end) {
-		printf("Disassembled %d (of active %d) CPU addresses.\n", addrs, active);
+		fprintf(stderr, "Disassembled %d (of active %d) CPU addresses.\n",
+			addrs, active);
 	} else {
-		printf("Disassembled last %d (of active %d) CPU addresses, wrapping...\n", addrs, active);
+		fprintf(stderr, "Disassembled last %d (of active %d) CPU addresses, wrapping...\n",
+			addrs, active);
 		nextpc = 0;
 	}
 	/* restore disassembly columns */
@@ -539,7 +531,6 @@ static void leave_instruction_column(int *oldcols)
 	Disasm_SetColumns(newcols);
 }
 
-#if ENABLE_WINUAE_CPU
 /**
  * compare function for qsort() to sort CPU profile data by instruction cache misses.
  */
@@ -579,17 +570,17 @@ void Profile_CpuShowInstrMisses(int show)
 
 	leave_instruction_column(oldcols);
 
-	printf("addr:\t\ti-cache misses:\n");
+	fprintf(stderr, "addr:\t\ti-cache misses:\n");
 	show = (show < active ? show : active);
 	for (end = sort_arr + show; sort_arr < end; sort_arr++) {
 		addr = index2address(*sort_arr);
 		count = data[*sort_arr].i_misses;
 		percentage = 100.0*count/cpu_profile.all.i_misses;
-		printf("0x%06x\t%5.2f%%\t%d%s\t", addr, percentage, count,
+		fprintf(stderr, "0x%06x\t%5.2f%%\t%d%s\t", addr, percentage, count,
 		       count == MAX_CPU_PROFILE_VALUE ? " (OVERFLOW)" : "");
-		Disasm(stdout, addr, &nextpc, 1);
+		Disasm(stderr, addr, &nextpc, 1);
 	}
-	printf("%d CPU addresses listed.\n", show);
+	fprintf(stderr, "%d CPU addresses listed.\n", show);
 
 	Disasm_SetColumns(oldcols);
 }
@@ -633,30 +624,21 @@ void Profile_CpuShowDataHits(int show)
 
 	leave_instruction_column(oldcols);
 
-	printf("addr:\t\td-cache hits:\n");
+	fprintf(stderr, "addr:\t\td-cache hits:\n");
 	show = (show < active ? show : active);
 	for (end = sort_arr + show; sort_arr < end; sort_arr++) {
 		addr = index2address(*sort_arr);
 		count = data[*sort_arr].d_hits;
 		percentage = 100.0*count/cpu_profile.all.d_hits;
-		printf("0x%06x\t%5.2f%%\t%d%s\t", addr, percentage, count,
+		fprintf(stderr, "0x%06x\t%5.2f%%\t%d%s\t", addr, percentage, count,
 		       count == MAX_CPU_PROFILE_VALUE ? " (OVERFLOW)" : "");
-		Disasm(stdout, addr, &nextpc, 1);
+		Disasm(stderr, addr, &nextpc, 1);
 	}
-	printf("%d CPU addresses listed.\n", show);
+	fprintf(stderr, "%d CPU addresses listed.\n", show);
 
 	Disasm_SetColumns(oldcols);
 }
 
-#else
-void Profile_CpuShowInstrMisses(int show) {
-	fprintf(stderr, "Cache information is recorded only with WinUAE CPU.\n");
-}
-void Profile_CpuShowDataHits(int show) {
-	fprintf(stderr, "Cache information is recorded only with WinUAE CPU.\n");
-}
-#endif
-
 
 /**
  * compare function for qsort() to sort CPU profile data by cycles counts.
@@ -697,17 +679,17 @@ void Profile_CpuShowCycles(int show)
 
 	leave_instruction_column(oldcols);
 
-	printf("addr:\t\tcycles:\n");
+	fprintf(stderr, "addr:\t\tcycles:\n");
 	show = (show < active ? show : active);
 	for (end = sort_arr + show; sort_arr < end; sort_arr++) {
 		addr = index2address(*sort_arr);
 		count = data[*sort_arr].cycles;
 		percentage = 100.0*count/cpu_profile.all.cycles;
-		printf("0x%06x\t%5.2f%%\t%d%s\t", addr, percentage, count,
+		fprintf(stderr, "0x%06x\t%5.2f%%\t%d%s\t", addr, percentage, count,
 		       count == MAX_CPU_PROFILE_VALUE ? " (OVERFLOW)" : "");
-		Disasm(stdout, addr, &nextpc, 1);
+		Disasm(stderr, addr, &nextpc, 1);
 	}
-	printf("%d CPU addresses listed.\n", show);
+	fprintf(stderr, "%d CPU addresses listed.\n", show);
 
 	Disasm_SetColumns(oldcols);
 }
@@ -756,17 +738,17 @@ void Profile_CpuShowCounts(int show, bool only_symbols)
 
 	if (!only_symbols) {
 		leave_instruction_column(oldcols);
-		printf("addr:\t\tcount:\n");
+		fprintf(stderr, "addr:\t\tcount:\n");
 		for (end = sort_arr + show; sort_arr < end; sort_arr++) {
 			addr = index2address(*sort_arr);
 			count = data[*sort_arr].count;
 			percentage = 100.0*count/cpu_profile.all.count;
-			printf("0x%06x\t%5.2f%%\t%d%s\t",
+			fprintf(stderr, "0x%06x\t%5.2f%%\t%d%s\t",
 			       addr, percentage, count,
 			       count == MAX_CPU_PROFILE_VALUE ? " (OVERFLOW)" : "");
-			Disasm(stdout, addr, &nextpc, 1);
+			Disasm(stderr, addr, &nextpc, 1);
 		}
-		printf("%d CPU addresses listed.\n", show);
+		fprintf(stderr, "%d CPU addresses listed.\n", show);
 		Disasm_SetColumns(oldcols);
 		return;
 	}
@@ -780,7 +762,7 @@ void Profile_CpuShowCounts(int show, bool only_symbols)
 
 	leave_instruction_column(oldcols);
 
-	printf("addr:        %%:   count:  symbol:                    disassembly:\n");
+	fprintf(stderr, "addr:        %%:   count:  symbol:                    disassembly:\n");
 	for (end = sort_arr + active; sort_arr < end; sort_arr++) {
 
 		addr = index2address(*sort_arr);
@@ -790,17 +772,17 @@ void Profile_CpuShowCounts(int show, bool only_symbols)
 		}
 		count = data[*sort_arr].count;
 		percentage = 100.0*count/cpu_profile.all.count;
-		printf("0x%06x %6.2f %8d  %-26s %s",
+		fprintf(stderr, "0x%06x %6.2f %8d  %-26s %s",
 		       addr, percentage, count, name,
 		       count == MAX_CPU_PROFILE_VALUE ? "(OVERFLOW) " : "");
-		Disasm(stdout, addr, &nextpc, 1);
+		Disasm(stderr, addr, &nextpc, 1);
 
 		matched++;
 		if (matched >= show || matched >= symbols) {
 			break;
 		}
 	}
-	printf("%d CPU symbols listed.\n", matched);
+	fprintf(stderr, "%d CPU symbols listed.\n", matched);
 
 	Disasm_SetColumns(oldcols);
 }
@@ -828,11 +810,21 @@ void Profile_CpuSave(FILE *out)
 {
 	Uint32 text, end;
 	fputs("Field names:\tExecuted instructions, Used cycles, Instruction cache misses, Data cache hits\n", out);
-	/* (Python) regexp that matches address and all described fields from disassembly:
-	 * $<hex>  :  <ASM>  <percentage>% (<count>, <cycles>, <i-misses>, <d-hits>)
-	 * $e5af38 :   rts           0.00% (12, 0, 12, 0)
-	 */
-	fputs("Field regexp:\t^\\$([0-9a-f]+) :.*% \\((.*)\\)$\n", out);
+
+	/* (Python) regexp that matches address & profiling data fields from the disassembly */
+	if (ConfigureParams.Debugger.bDisasmUAE) {
+		 /* WinAUE CPU core disassembler (current default) output:
+		  * <addr> <code>  <ASM>     <percentage>% (<count>, <cycles>, <i-misses>, <d-hits>)
+		  * 00e00cfe 4e75  rts  == $e66218   0.16% (48753, 780396, 0, 0)
+		  */
+		fputs("Field regexp:\t^([0-9a-f]+) .*% \\((.*)\\)$\n", out);
+	} else {
+		 /* External 68kDisass disassembler (earlier default) output:
+		  * $<addr>  :  <ASM>  <percentage>% (<count>, <cycles>, <i-misses>, <d-hits>)
+		  * $e5af38 :   rts           0.00% (12, 0, 12, 0)
+		  */
+		fputs("Field regexp:\t^\\$([0-9a-f]+) :.*% \\((.*)\\)$\n", out);
+	}
 	/* some information for interpreting the addresses */
 	fprintf(out, "ST_RAM:\t\t0x%06x-0x%06x\n", 0, STRamEnd);
 	end = TosAddress + TosSize;
@@ -868,7 +860,7 @@ bool Profile_CpuStart(void)
 		free(cpu_profile.data);
 		cpu_profile.sort_arr = NULL;
 		cpu_profile.data = NULL;
-		printf("Freed previous CPU profile buffers.\n");
+		fprintf(stderr, "Freed previous CPU profile buffers.\n");
 	}
 	if (!cpu_profile.enabled) {
 		return false;
@@ -890,7 +882,7 @@ bool Profile_CpuStart(void)
 		perror("ERROR, new CPU profile buffer alloc failed");
 		return false;
 	}
-	printf("Allocated CPU profile buffer (%d MB).\n",
+	fprintf(stderr, "Allocated CPU profile buffer (%d MB).\n",
 	       (int)sizeof(*cpu_profile.data)*size/(1024*1024));
 	cpu_profile.size = size;
 
@@ -1105,7 +1097,6 @@ static void log_last_loop(void)
 	}
 }
 
-# if DEBUG || ENABLE_WINUAE_CPU
 /**
  * Warning for values going out of expected range
  */
@@ -1117,7 +1108,7 @@ static Uint32 warn_too_large(const char *name, const int value, const int limit,
 		Disasm(stderr, prev_pc, &nextpc, 1);
 		Disasm(stderr, pc, &nextpc, 1);
 		if (cpu_warnings.largevalue == MAX_SHOW_COUNT) {
-			fprintf(stderr, "Further warnings won't be shown.\n");
+			fprintf(stderr, "Further warnings will not be shown.\n");
 		}
 	}
 #if DEBUG
@@ -1126,7 +1117,6 @@ static Uint32 warn_too_large(const char *name, const int value, const int limit,
 #endif
 	return limit - 1;
 }
-#endif
 
 /**
  * Update CPU cycle and count statistics for PC address.
@@ -1139,11 +1129,7 @@ void Profile_CpuUpdate(void)
 	counters_t *counters = &(cpu_profile.all);
 	Uint32 pc, prev_pc, idx, cycles;
 	cpu_profile_item_t *prev;
-#if ENABLE_WINUAE_CPU
 	Uint32 i_hits, d_hits, i_misses, d_misses;
-#else
-	const Uint32 i_misses = 0, d_hits = 0;
-#endif
 
 	prev_pc = cpu_profile.prev_pc;
 	/* PC may have extra bits when using 24 bit addressing, they need to be masked away as
@@ -1188,7 +1174,6 @@ void Profile_CpuUpdate(void)
 		prev->cycles = MAX_CPU_PROFILE_VALUE;
 	}
 
-#if ENABLE_WINUAE_CPU
 	/* only WinUAE CPU core provides cache information */
 	i_hits = CpuInstruction.I_Cache_hit;
 	d_hits = CpuInstruction.D_Cache_hit;
@@ -1248,7 +1233,6 @@ void Profile_CpuUpdate(void)
 		d_misses = warn_too_large("number of CPU data cache misses", d_misses, MAX_D_MISSES, prev_pc, pc);
 	}
 	cpu_profile.d_miss_counts[d_misses]++;
-#endif   /* ENABLE_WINUAE_CPU */
 
 	if (cpu_callinfo.sites) {
 		collect_calls(prev_pc, counters);
@@ -1269,7 +1253,7 @@ void Profile_CpuUpdate(void)
 			fputs("WARNING: instruction opcode family is zero (=i_ILLG) for instruction:\n", stderr);
 			Disasm(stderr, prev_pc, &nextpc, 1);
 			if (cpu_warnings.opfamily == MAX_SHOW_COUNT) {
-				fprintf(stderr, "Further warnings won't be shown.\n");
+				fprintf(stderr, "Further warnings will not be shown.\n");
 			}
 		}
 	}
@@ -1277,24 +1261,6 @@ void Profile_CpuUpdate(void)
 	if (unlikely(cycles > 512 && OpcodeFamily != i_STOP)) {
 		warn_too_large("cycles", cycles, 512, prev_pc, pc);
 	}
-# if !ENABLE_WINUAE_CPU
-	{
-		static Uint32 prev_cycles = 0, prev_pc2 = 0;
-		if (unlikely(cycles == 0 && prev_cycles == 0)) {
-			if (++cpu_warnings.zerocycles <= MAX_SHOW_COUNT) {
-				Uint32 nextpc;
-				fputs("WARNING: Zero cycles for successive opcodes:\n", stderr);
-				Disasm(stderr, prev_pc2, &nextpc, 1);
-				Disasm(stderr, prev_pc, &nextpc, 1);
-				if (cpu_warnings.zerocycles == MAX_SHOW_COUNT) {
-					fprintf(stderr, "Further warnings won't be shown.\n");
-				}
-			}
-		}
-		prev_cycles = cycles;
-		prev_pc2 = prev_pc;
-	}
-# endif
 #endif
 }
 
@@ -1435,7 +1401,7 @@ void Profile_CpuStop(void)
 		cpu_profile.data = NULL;
 		return;
 	}
-	printf("Allocated CPU profile address buffer (%d KB).\n",
+	fprintf(stderr, "Allocated CPU profile address buffer (%d KB).\n",
 	       (int)sizeof(*sort_arr)*(active+512)/1024);
 	cpu_profile.sort_arr = sort_arr;
 	cpu_profile.active = active;
