@@ -139,6 +139,19 @@ Uint32 Vars_GetVdiOpcode(void)
 	return INVALID_OPCODE;
 }
 
+/** return 1 if PC is on Symbol, 0 otherwise
+ */
+static Uint32 PConSymbol(void)
+{
+	const char *sym;
+	Uint32 pc = M68000_GetPC();
+	sym = Symbols_GetByCpuAddress(pc, SYMTYPE_TEXT);
+	if (sym) {
+		return 1;
+	}
+	return 0;
+}
+
 /** return first word in OS call parameters
  */
 static Uint32 GetOsCallParam(void)
@@ -158,25 +171,28 @@ static const var_addr_t hatari_vars[] = {
 	{ "Basepage", (Uint32*)DebugInfo_GetBASEPAGE, VALUE_TYPE_FUNCTION32, 0, "invalid before Desktop is up" },
 	{ "BiosOpcode", (Uint32*)GetBiosOpcode, VALUE_TYPE_FUNCTION32, 16, "$FFFF when not on BIOS trap" },
 	{ "BSS", (Uint32*)DebugInfo_GetBSS, VALUE_TYPE_FUNCTION32, 0, "invalid before Desktop is up" },
+	{ "CpuCallDepth", (Uint32*)DebugCpu_CallDepth, VALUE_TYPE_FUNCTION32, 0, NULL }, /* call depth for 'next subreturn' */
 	{ "CpuInstr", (Uint32*)DebugCpu_InstrCount, VALUE_TYPE_FUNCTION32, 0, "CPU instructions count" },
-	{ "CpuOpcodeType", (Uint32*)DebugCpu_OpcodeType, VALUE_TYPE_FUNCTION32, 0, "internal CPU instruction type" },
+	{ "CpuOpcodeType", (Uint32*)DebugCpu_OpcodeType, VALUE_TYPE_FUNCTION32, 0, NULL }, /* opcode type for 'next' */
 	{ "CycleCounter", (Uint32*)GetCycleCounter, VALUE_TYPE_FUNCTION32, 0, "global cycles counter (lower 32 bits)" },
 	{ "DATA", (Uint32*)DebugInfo_GetDATA, VALUE_TYPE_FUNCTION32, 0, "invalid before Desktop is up" },
 #if ENABLE_DSP_EMU
+	{ "DspCallDepth", (Uint32*)DebugDsp_CallDepth, VALUE_TYPE_FUNCTION32, 0, NULL }, /* call depth for 'dspnext subreturn' */
 	{ "DspInstr", (Uint32*)DebugDsp_InstrCount, VALUE_TYPE_FUNCTION32, 0, "DSP instructions count" },
-	{ "DspOpcodeType", (Uint32*)DebugDsp_OpcodeType, VALUE_TYPE_FUNCTION32, 0, "internal DSP instruction type" },
+	{ "DspOpcodeType", (Uint32*)DebugDsp_OpcodeType, VALUE_TYPE_FUNCTION32, 0, NULL }, /* opcode type for 'dspnext' */
 #endif
-	{ "FrameCycles", (Uint32*)GetFrameCycles, VALUE_TYPE_FUNCTION32, 0, NULL },
+	{ "FrameCycles", (Uint32*)GetFrameCycles, VALUE_TYPE_FUNCTION32, 0, "cycles since VBL" },
 	{ "GemdosOpcode", (Uint32*)GetGemdosOpcode, VALUE_TYPE_FUNCTION32, 16, "$FFFF when not on GEMDOS trap" },
-	{ "HBL", (Uint32*)&nHBL, VALUE_TYPE_VAR32, sizeof(nHBL)*8, NULL },
+	{ "HBL", (Uint32*)&nHBL, VALUE_TYPE_VAR32, sizeof(nHBL)*8, "number of HBL interrupts"  },
 	{ "LineAOpcode", (Uint32*)GetLineAOpcode, VALUE_TYPE_FUNCTION32, 16, "$FFFF when not on Line-A opcode" },
-	{ "LineCycles", (Uint32*)GetLineCycles, VALUE_TYPE_FUNCTION32, 0, "is always divisible by 4" },
+	{ "LineCycles", (Uint32*)GetLineCycles, VALUE_TYPE_FUNCTION32, 0, "cycles since HBL (divisible by 4)" },
 	{ "LineFOpcode", (Uint32*)GetLineFOpcode, VALUE_TYPE_FUNCTION32, 16, "$FFFF when not on Line-F opcode" },
-	{ "NextPC", (Uint32*)GetNextPC, VALUE_TYPE_FUNCTION32, 0, NULL },
-	{ "OsCallParam", (Uint32*)GetOsCallParam, VALUE_TYPE_FUNCTION32, 16, "word value, valid only on OS call opcode breakpoint" },
+	{ "NextPC", (Uint32*)GetNextPC, VALUE_TYPE_FUNCTION32, 0, "Next instruction address" },
+	{ "OsCallParam", (Uint32*)GetOsCallParam, VALUE_TYPE_FUNCTION32, 16, "valid only on OS call opcode breakpoint" },
+	{ "PConSymbol", (Uint32*)PConSymbol, VALUE_TYPE_FUNCTION32, 16, "1 if PC on symbol, 0 otherwise" },
 	{ "TEXT", (Uint32*)DebugInfo_GetTEXT, VALUE_TYPE_FUNCTION32, 0, "invalid before Desktop is up" },
 	{ "TEXTEnd", (Uint32*)DebugInfo_GetTEXTEnd, VALUE_TYPE_FUNCTION32, 0, "invalid before Desktop is up" },
-	{ "VBL", (Uint32*)&nVBLs, VALUE_TYPE_VAR32, sizeof(nVBLs)*8, NULL },
+	{ "VBL", (Uint32*)&nVBLs, VALUE_TYPE_VAR32, sizeof(nVBLs)*8, "number of VBL interrupts" },
 	{ "VdiOpcode", (Uint32*)Vars_GetVdiOpcode, VALUE_TYPE_FUNCTION32, 16, "$FFFF when not on VDI trap" },
 	{ "XbiosOpcode", (Uint32*)GetXbiosOpcode, VALUE_TYPE_FUNCTION32, 16, "$FFFF when not on XBIOS trap" }
 };
@@ -278,17 +294,30 @@ bool Vars_GetVariableValue(const char *name, Uint32 *value)
 int Vars_List(int nArgc, char *psArgv[])
 {
 	Uint32 value;
-	int i;
-	fputs("Hatari debugger builtin symbols and their values are:\n", stderr);
+	char numstr[16];
+	int i, maxlen = 0;
 	for (i = 0; i < ARRAY_SIZE(hatari_vars); i++) {
-		const var_addr_t *hvar = hatari_vars + i;
-		value = Vars_GetValue(hvar);
-		fprintf(stderr, "  - %s: $%X / #%d", hvar->name, value, value);
-		if (hvar->info) {
-			fprintf(stderr, " -- %s\n", hvar->info);
-		} else {
-			fprintf(stderr, "\n");
+		int len = strlen(hatari_vars[i].name);
+		if (len > maxlen) {
+			maxlen = len;
 		}
 	}
+	fputs("Hatari debugger builtin symbols and their values are:\n", stderr);
+	for (i = 0; i < ARRAY_SIZE(hatari_vars); i++) {
+		const var_addr_t *hvar = &hatari_vars[i];
+		if (!hvar->info) {
+			/* debugger internal variables don't have descriptions */
+			continue;
+		}
+		value = Vars_GetValue(hvar);
+		if (hvar->bits == 16) {
+			fprintf(stderr, " %*s:     $%04X", maxlen, hvar->name, value);
+		} else {
+			fprintf(stderr, " %*s: $%08X", maxlen, hvar->name, value);
+		}
+		sprintf(numstr, "(%d)", value);
+		fprintf(stderr, " %-*s %s\n", 12, numstr, hvar->info);
+	}
+	fputs("Some of the variables are valid only in specific situations.\n", stderr);
 	return DEBUGGER_CMDDONE;
 }
