@@ -34,8 +34,8 @@
     $FF8911 (byte) : Frame End Mi
     $FF8913 (byte) : Frame End Lo
     $FF8920 (word) : Sound Mode Control (frequency, mono/stereo)
-    $FF8922 (byte) : Microwire Data Register
-    $FF8924 (byte) : Microwire Mask Register
+    $FF8922 (word) : Microwire Data Register
+    $FF8924 (word) : Microwire Mask Register
 
   
   The Microwire and LMC 1992 commands :
@@ -99,6 +99,7 @@ const char DmaSnd_fileid[] = "Hatari dmaSnd.c";
 #include "audio.h"
 #include "configuration.h"
 #include "dmaSnd.h"
+#include "cycles.h"
 #include "cycInt.h"
 #include "ioMem.h"
 #include "log.h"
@@ -110,6 +111,7 @@ const char DmaSnd_fileid[] = "Hatari dmaSnd.c";
 #include "screen.h"
 #include "video.h"
 #include "m68000.h"
+#include "clocks_timings.h"
 
 #define TONE_STEPS 13
 
@@ -303,6 +305,18 @@ static void DmaSnd_Update_XSINT_Line ( Uint8 Bit )
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Return the value of the XSINT line
+ *  0=dma sound is idle
+ *  1=dma sound is playing
+ */
+Uint8	DmaSnd_Get_XSINT_Line ( void )
+{
+	return dma.XSINT_Signal;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function is called on every HBL to ensure the DMA Audio's FIFO
@@ -331,10 +345,10 @@ static void DmaSnd_FIFO_Refill(void)
 	{
 		/* Add one word to the FIFO */
 		LOG_TRACE(TRACE_DMASND, "DMA snd fifo refill adr=%x pos %d nb %d %x %x\n", dma.frameCounterAddr , dma.FIFO_Pos , dma.FIFO_NbBytes ,
-			STRam[ dma.frameCounterAddr ] , STRam[ dma.frameCounterAddr+1 ] );
+			STMemory_DMA_ReadByte ( dma.frameCounterAddr ) , STMemory_DMA_ReadByte ( dma.frameCounterAddr+1 ) );
 
-		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+0 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STRam[ dma.frameCounterAddr ];	/* add upper byte of the word */
-		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+1 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STRam[ dma.frameCounterAddr+1 ];	/* add lower byte of the word */
+		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+0 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STMemory_DMA_ReadByte ( dma.frameCounterAddr );	/* add upper byte of the word */
+		dma.FIFO[ ( dma.FIFO_Pos+dma.FIFO_NbBytes+1 ) & DMASND_FIFO_SIZE_MASK ] = (Sint8)STMemory_DMA_ReadByte ( dma.frameCounterAddr+1 );	/* add lower byte of the word */
 
 		dma.FIFO_NbBytes += 2;				/* One word more in the FIFO */
 
@@ -530,20 +544,20 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 	{
 		for (i = 0; i < nSamplesToGenerate; i++)
 		{
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = AudioMixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = AudioMixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* mixing=0 DMA only */
 					/* mixing=2 DMA and input 2 (YM2149 LPF) -> DMA */
 					/* mixing=3 DMA and input 3 -> DMA */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 			}
 		}
@@ -580,22 +594,22 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 				DmaInitSample = false;
 			}
 
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = AudioMixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* mixing=0 DMA only */
 					/* mixing=2 DMA and input 2 (YM2149 LPF) -> DMA */
 					/* mixing=3 DMA and input 3 -> DMA */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
 					break;
 			}
 
-			MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][0];		/* right = left */
+			AudioMixBuffer[nBufIdx][1] = AudioMixBuffer[nBufIdx][0];	/* right = left */
 
 			/* Increase freq counter */
 			frameCounter_float += FreqRatio;
@@ -624,20 +638,20 @@ void DmaSnd_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 				DmaInitSample = false;
 			}
 
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 			switch (microwire.mixing) {
 				case 1:
 					/* DMA and YM2149 mixing */
-					MixBuffer[nBufIdx][0] = MixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = MixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = AudioMixBuffer[nBufIdx][0] + dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = AudioMixBuffer[nBufIdx][1] + dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 				default:
 					/* mixing=0 DMA only */
 					/* mixing=2 DMA and input 2 (YM2149 LPF) -> DMA */
 					/* mixing=3 DMA and input 3 -> DMA */
-					MixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
-					MixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][0] = dma.FrameLeft * -((256*3/4)/4)/4;
+					AudioMixBuffer[nBufIdx][1] = dma.FrameRight * -((256*3/4)/4)/4;
 					break;
 			}
 
@@ -675,21 +689,21 @@ static void DmaSnd_Apply_LMC(int nMixBufIdx, int nSamplesToGenerate)
 
 	/* Apply LMC1992 sound modifications (Left, Right and Master Volume) */
 	for (i = 0; i < nSamplesToGenerate; i++) {
-		nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+		nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
-		sample = DmaSnd_IIRfilterL( Subsonic_IIR_HPF_Left( MixBuffer[nBufIdx][0]));
+		sample = DmaSnd_IIRfilterL( Subsonic_IIR_HPF_Left( AudioMixBuffer[nBufIdx][0]));
 		if (sample<-32767)						/* check for overflow to clip waveform */
 			sample = -32767;
 		else if (sample>32767)
 			sample = 32767;
-		MixBuffer[nBufIdx][0] = sample;
+		AudioMixBuffer[nBufIdx][0] = sample;
 
-		sample = DmaSnd_IIRfilterR( Subsonic_IIR_HPF_Right(MixBuffer[nBufIdx][1]));
+		sample = DmaSnd_IIRfilterR( Subsonic_IIR_HPF_Right(AudioMixBuffer[nBufIdx][1]));
 		if (sample<-32767)						/* check for overflow to clip waveform */
 			sample = -32767;
 		else if (sample>32767)
 			sample = 32767;
-		MixBuffer[nBufIdx][1] = sample;
+		AudioMixBuffer[nBufIdx][1] = sample;
  	}
 }
 
@@ -714,7 +728,7 @@ void DmaSnd_STE_HBL_Update(void)
 
 	/* If DMA sound is ON or FIFO is not empty, update sound */
 	if  ( (nDmaSoundControl & DMASNDCTRL_PLAY) || ( dma.FIFO_NbBytes > 0 ) )
-		Sound_Update(false);
+		Sound_Update ( CyclesGlobalClockCounter );
 
 	/* As long as display is OFF, the DMA will refill the FIFO after playing some samples during the HBL */
 	DmaSnd_FIFO_Refill ();
@@ -730,7 +744,7 @@ static Uint32 DmaSnd_GetFrameCount(void)
 	Uint32 nActCount;
 
 	/* Update sound to get the current DMA frame address */
-	Sound_Update(false);
+	Sound_Update ( CyclesGlobalClockCounter );
 
 	if (nDmaSoundControl & DMASNDCTRL_PLAY)
 		nActCount = dma.frameCounterAddr;
@@ -751,8 +765,8 @@ void DmaSnd_SoundControl_ReadWord(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd control read: 0x%04x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			nDmaSoundControl,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -770,15 +784,15 @@ void DmaSnd_SoundControl_WriteWord(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd control write: 0x%04x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8900),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
 	}
 
-        /* Before starting/stopping DMA sound, create samples up until this point with current values */
-	Sound_Update(false);
+	/* Before starting/stopping DMA sound, create samples up until this point with current values */
+	Sound_Update ( Cycles_GetClockCounterOnWriteAccess() );
 
 	DMASndCtrl_old = nDmaSoundControl;
 	nDmaSoundControl = IoMem_ReadWord(0xff8900) & 3;
@@ -836,8 +850,8 @@ void DmaSnd_FrameStartHigh_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame start high: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8903) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -851,8 +865,8 @@ void DmaSnd_FrameStartMed_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame start med: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8905) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -863,8 +877,8 @@ void DmaSnd_FrameStartLow_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame start low: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8907) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -878,8 +892,8 @@ void DmaSnd_FrameCountHigh_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame count high: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8909) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -893,8 +907,8 @@ void DmaSnd_FrameCountMed_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame count med: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff890b) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -905,8 +919,8 @@ void DmaSnd_FrameCountLow_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame count low: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff890d) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -917,8 +931,8 @@ void DmaSnd_FrameEndHigh_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
  		LOG_TRACE_PRINT("DMA snd frame end high: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff890f) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -932,8 +946,8 @@ void DmaSnd_FrameEndMed_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame end med: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8911) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -944,8 +958,8 @@ void DmaSnd_FrameEndLow_WriteByte(void)
 {
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd frame end low: 0x%02x at pos %d/%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadByte(0xff8913) , dma.frameCounterAddr - dma.frameStartAddr , dma.frameEndAddr - dma.frameStartAddr  ,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -966,8 +980,8 @@ void DmaSnd_SoundModeCtrl_ReadByte(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd mode read: 0x%02x video_cyc=%d %d@%d pc=%x instr_cycle %d\n", dma.soundMode,
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
 	}
@@ -986,8 +1000,8 @@ void DmaSnd_SoundModeCtrl_WriteByte(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("DMA snd mode write: 0x%02x mode=%s freq=%d video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			SoundModeNew, SoundModeNew & DMASNDMODE_MONO ? "mono" : "stereo" , DmaSndSampleRates[ SoundModeNew & 3 ],
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1160,8 +1174,8 @@ void DmaSnd_MicrowireData_ReadWord(void)
 	/* Shifting is done in DmaSnd_InterruptHandler_Microwire! */
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire data read: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8922),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1186,8 +1200,8 @@ void DmaSnd_MicrowireData_WriteWord(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire data write: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8922),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1203,8 +1217,8 @@ void DmaSnd_MicrowireMask_ReadWord(void)
 	/* Same as with data register, but mask is rotated, not shifted. */
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire mask read: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8924),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1225,8 +1239,8 @@ void DmaSnd_MicrowireMask_WriteWord(void)
 
 	if(LOG_TRACE_LEVEL(TRACE_DMASND))
 	{
-                int FrameCycles, HblCounterVideo, LineCycles;
-                Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
+		int FrameCycles, HblCounterVideo, LineCycles;
+		Video_GetPosition ( &FrameCycles , &HblCounterVideo , &LineCycles );
 		LOG_TRACE_PRINT("Microwire mask write: 0x%x video_cyc=%d %d@%d pc=%x instr_cycle %d\n",
 			IoMem_ReadWord(0xff8924),
 			FrameCycles, LineCycles, HblCounterVideo, M68000_GetPC(), CurrentInstrCycles);
@@ -1468,6 +1482,6 @@ void DmaSnd_Info(FILE *fp, Uint32 dummy)
 		return;
 	}
 	fprintf(fp, "\n");
-	fprintf(fp, "$FF8922.b : Microwire Data     : %02x\n", IoMem_ReadByte(0xff8922));
-	fprintf(fp, "$FF8922.b : Microwire Mask     : %02x\n", IoMem_ReadByte(0xff8924));
+	fprintf(fp, "$FF8922.w : Microwire Data     : %04x\n", IoMem_ReadWord(0xff8922));
+	fprintf(fp, "$FF8924.w : Microwire Mask     : %04x\n", IoMem_ReadWord(0xff8924));
 }
