@@ -94,6 +94,7 @@ const char crossbar_fileid[] = "Hatari Crossbar.c";
 #include "main.h"
 #include "audio.h"
 #include "configuration.h"
+#include "cycles.h"
 #include "cycInt.h"
 #include "dmaSnd.h"
 #include "m68000.h"
@@ -224,6 +225,7 @@ struct dma_s {
 	Uint32 isConnectedToDspInHandShakeMode;
 	Uint32 isConnectedToDma;
 	Uint32 handshakeMode_Frame;	/* state of the frame in handshake mode */
+	Uint32 handshakeMode_masterClk;	/* 0 = crossbar master clock ; 1 = DSP master clock */
 };
 
 struct crossbar_s {
@@ -269,7 +271,8 @@ struct crossbar_s {
 
 	Uint32 save_special_transfer;		/* Used in a special undocumented transfer mode (dsp sent is not in handshake mode and dsp receive is in handshake mode) */
 
-	Uint8  SNDINT_Signal;		/* Value of the SNDINT/SOUNDINT signal (connected to MFP) */
+	Uint8  SNDINT_Signal;		/* Value of the SNDINT signal (connected to MFP's GPIP7) */
+	Uint8  SOUNDINT_Signal;		/* Value of the SOUNDINT signal (connected to MFP's Timer A input) */
 };
 
 struct codec_s {
@@ -315,16 +318,19 @@ void Crossbar_Reset(bool bCold)
 	dmaPlay.currentFrame = 0;
 	dmaPlay.isConnectedToDspInHandShakeMode = 0;
 	dmaPlay.handshakeMode_Frame = 0;
+	dmaPlay.handshakeMode_masterClk = 0;
 	dmaRecord.isRunning = 0;
 	dmaRecord.loopMode = 0;
 	dmaRecord.currentFrame = 0;
 	dmaRecord.isConnectedToDspInHandShakeMode = 0;
 	dmaRecord.handshakeMode_Frame = 0;
+	dmaRecord.handshakeMode_masterClk = 0;
 
-	/* DMA stopped, force SNDINT to 0/LOW */
+	/* DMA stopped, force SNDINT/SOUNDINT to 0/LOW */
 	crossbar.SNDINT_Signal = MFP_GPIP_STATE_LOW;
-	MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE7 , MFP_GPIP_STATE_LOW );
-	MFP_TimerA_Set_Line_Input ( pMFP_Main , MFP_GPIP_STATE_LOW );
+	crossbar.SOUNDINT_Signal = MFP_GPIP_STATE_LOW;
+	MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE7 , crossbar.SNDINT_Signal );
+	MFP_TimerA_Set_Line_Input ( pMFP_Main , crossbar.SOUNDINT_Signal );
 
 
 	/* DAC inits */
@@ -427,7 +433,7 @@ void Crossbar_MemorySnapShot_Capture(bool bSave)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Update the value of the SNDINT/SOUNDINT line ; this line is connected to TAI and to GPIP7
+ * Update the value of the SNDINT/SOUNDINT lines ; these line are connected to TAI and to GPIP7
  * Depending on the transition, this can trigger MFP interrupt for Timer A or for GPIP7
  *  - Bit is set to 0/LOW when dma sound is idle
  *  - Bit is set to 1/HIGH when dma sound is playing / recording
@@ -445,7 +451,7 @@ static void Crossbar_Update_SNDINT_Line ( bool RecordMode , Uint8 Bit )
 
 		/* Send a TimerA_Int at end of replay buffer if enabled */
 		if (dmaPlay.timerA_int) {
-			crossbar.SNDINT_Signal = Bit;
+			crossbar.SOUNDINT_Signal = Bit;
 			MFP_TimerA_Set_Line_Input ( pMFP_Main , Bit );			/* Update events count / interrupt for timer A if needed */
 			LOG_TRACE(TRACE_CROSSBAR, "Crossbar : MFP Timer A interrupt from DMA play\n");
 		}
@@ -462,14 +468,21 @@ static void Crossbar_Update_SNDINT_Line ( bool RecordMode , Uint8 Bit )
 
 		/* Send a TimerA_Int at end of record buffer if enabled */
 		if (dmaRecord.timerA_int) {
-			crossbar.SNDINT_Signal = Bit;
+			crossbar.SOUNDINT_Signal = Bit;
 			MFP_TimerA_Set_Line_Input ( pMFP_Main , Bit );			/* Update events count / interrupt for timer A if needed */
 			LOG_TRACE(TRACE_CROSSBAR, "Crossbar : MFP Timer A interrupt from DMA record\n");
 		}
 	}
+}
 
-// 	MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE7 , Bit );
-// 	MFP_TimerA_Set_Line_Input ( pMFP_Main , Bit );			/* Update events count / interrupt for timer A if needed */
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Return the value of the SNDINT line, used to update MFP's GPIP7
+ */
+Uint8 Crossbar_Get_SNDINT_Line (void)
+{
+	return crossbar.SNDINT_Signal;
 }
 
 
@@ -549,7 +562,7 @@ void Crossbar_DmaCtrlReg_WriteByte(void)
 	else if (dmaPlay.isRunning && ((sndCtrl & CROSSBAR_SNDCTRL_PLAY) == 0))
 	{
 		/* Create samples up until this point with current values */
-		Sound_Update(false);
+		Sound_Update ( Cycles_GetClockCounterOnWriteAccess() );
 
 		/* Turning off DMA play sound emulation */
 		dmaPlay.isRunning = 0;
@@ -1055,6 +1068,7 @@ void Crossbar_DstControler_WriteWord(void)
 
 	dmaPlay.isConnectedToDspInHandShakeMode = (((destCtrl >> 4) & 7) == 0 ? 1 : 0);
 	dmaPlay.handshakeMode_Frame = dmaPlay.isConnectedToDspInHandShakeMode;
+	dmaPlay.handshakeMode_masterClk = 0;
 
 	dmaRecord.isConnectedToDspInHandShakeMode = ((destCtrl & 0xf) == 2 ? 1 : 0);
 }
@@ -1182,17 +1196,19 @@ void Crossbar_Recalculate_Clocks_Cycles(void)
 
 	/* Calculate 25 Mhz clock cycles */
 	/* Take nCpuFreqShift into account to keep a constant sound rate at all cpu freq */
-	cyclesClk = ((double)( ( CPU_FREQ << nCpuFreqShift ) ) / Crossbar_DetectSampleRate(25)) / (double)(crossbar.playTracks) / 2.0;
+	cyclesClk = ((double)( MachineClocks.CPU_Freq_Emul ) / Crossbar_DetectSampleRate(25)) / (double)(crossbar.playTracks) / 2.0;
 
 	crossbar.clock25_cycles = (int)(cyclesClk);
 	crossbar.clock25_cycles_decimal = (int)((cyclesClk - (double)(crossbar.clock25_cycles)) * (double)DECIMAL_PRECISION);
+//fprintf ( stderr , "freq_25=%d cyclesclk=%f cyc_int=%d cyc_float=%d\n",Crossbar_DetectSampleRate(25), cyclesClk, crossbar.clock25_cycles, crossbar.clock25_cycles_decimal);
 
 	/* Calculate 32 Mhz clock cycles */
 	/* Take nCpuFreqShift into account to keep a constant sound rate at all cpu freq */
-	cyclesClk = ((double)( ( CPU_FREQ << nCpuFreqShift ) ) / Crossbar_DetectSampleRate(32)) / (double)(crossbar.playTracks) / 2.0;
+	cyclesClk = ((double)( MachineClocks.CPU_Freq_Emul ) / Crossbar_DetectSampleRate(32)) / (double)(crossbar.playTracks) / 2.0;
 
 	crossbar.clock32_cycles = (int)(cyclesClk);
 	crossbar.clock32_cycles_decimal = (int)((cyclesClk - (double)(crossbar.clock32_cycles)) * (double)DECIMAL_PRECISION);
+//fprintf ( stderr , "freq_32=%d cyclesclk=%f cyc_int=%d cyc_float=%d\n",Crossbar_DetectSampleRate(32), cyclesClk, crossbar.clock25_cycles, crossbar.clock25_cycles_decimal);
 
 	LOG_TRACE(TRACE_CROSSBAR, "Crossbar : Recalculate_clock_Cycles\n");
 	LOG_TRACE(TRACE_CROSSBAR, "           clock25 : %d\n", crossbar.clock25_cycles);
@@ -1513,32 +1529,33 @@ static void Crossbar_Process_DMAPlay_Transfer(void)
 	if (dmaPlay.isRunning == 0)
 		return;
 
-	nFramePos = (dmaPlay.frameStartAddr + dmaPlay.frameCounter) % STRamEnd;
+	nFramePos = (dmaPlay.frameStartAddr + dmaPlay.frameCounter) & (DMA_MaskAddressHigh() << 16 | 0xffff);
 	increment_frame = 0;
 
 	/* 16 bits stereo mode ? */
 	if (crossbar.is16Bits) {
 		eightBits = 1;
-		value = (Sint16)do_get_mem_word(&STRam[nFramePos]);
+		value = (Sint16)STMemory_DMA_ReadWord(nFramePos);
 		increment_frame = 2;
 	}
 	/* 8 bits stereo ? */
 	else if (crossbar.isStereo) {
 		eightBits = 64;
-		value = (Sint8)STRam[nFramePos];
+		value = (Sint8)STMemory_DMA_ReadByte(nFramePos);
 		increment_frame = 1;
 	}
 	/* 8 bits mono */
 	else {
 		eightBits = 64;
-		value = (Sint8)STRam[nFramePos];
+		value = (Sint8)STMemory_DMA_ReadByte(nFramePos);
 		if ((dmaPlay.currentFrame & 1) == 0) {
 			increment_frame = 1;
 		}
 	}
 
 //fprintf ( stderr , "cbar %x %x %x\n" , dmaPlay.frameCounter , value , increment_frame );
-	if (dmaPlay.isConnectedToDspInHandShakeMode) {
+//	if (dmaPlay.isConnectedToDspInHandShakeMode && dmaPlay.handshakeMode_Frame != 0) {
+	if (dmaPlay.isConnectedToDspInHandShakeMode == 1 && dmaPlay.handshakeMode_masterClk == 1) {
 		/* Handshake mode */
 		if (dmaPlay.handshakeMode_Frame == 0)
 			return;
@@ -1602,7 +1619,7 @@ static void Crossbar_Process_DMAPlay_Transfer(void)
 		}
 		else {
 			/* Create samples up until this point with current values */
-			Sound_Update(false);
+			Sound_Update ( CyclesGlobalClockCounter );
 
 			dmaCtrlReg = IoMem_ReadByte(0xff8901) & 0xfe;
 			IoMem_WriteByte(0xff8901, dmaCtrlReg);
@@ -1619,6 +1636,7 @@ static void Crossbar_Process_DMAPlay_Transfer(void)
  * Function called when DmaPlay is in handshake mode */
 void Crossbar_DmaPlayInHandShakeMode(void)
 {
+	dmaPlay.handshakeMode_masterClk = 1;
 	dmaPlay.handshakeMode_Frame = 1;
 }
 
@@ -1659,23 +1677,23 @@ void Crossbar_SendDataToDmaRecord(Sint16 value)
 		return;
 	}
 
-	nFramePos = (dmaRecord.frameStartAddr + dmaRecord.frameCounter) % STRamEnd;
+	nFramePos = (dmaRecord.frameStartAddr + dmaRecord.frameCounter) & (DMA_MaskAddressHigh() << 16 | 0xffff);
 
 	/* 16 bits stereo mode ? */
 	if (crossbar.is16Bits) {
-		do_put_mem_word(&STRam[nFramePos], value);
+		STMemory_DMA_WriteWord(nFramePos, value);
 		dmaRecord.frameCounter += 2;
 	}
 	/* 8 bits stereo ? */
 	else if (crossbar.isStereo) {
-		do_put_mem_word(&STRam[nFramePos], value);
+		STMemory_DMA_WriteWord(nFramePos, value);
 		dmaRecord.frameCounter += 2;
 //		pFrameStart[dmaRecord.frameCounter] = (Uint8)value;
 //		dmaRecord.frameCounter ++;
 	}
 	/* 8 bits mono */
 	else {
-		STRam[nFramePos] = (Uint8)value;
+		STMemory_DMA_WriteByte(nFramePos, value);
 		dmaRecord.frameCounter ++;
 	}
 
@@ -1849,13 +1867,17 @@ void Crossbar_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 	Sint16 dac_read_left, dac_read_right;
 
 //fprintf ( stderr , "gen %03x %03x %03x %03x\n" , dac.writePosition , dac.readPosition , (dac.writePosition-dac.readPosition)%DACBUFFER_SIZE , nSamplesToGenerate );
+//fprintf ( stderr,  "codecAdcInput %d wordCount %d codecInputSource %d\n" , crossbar.codecAdcInput, dac.wordCount, crossbar.codecInputSource);
+//Uint32 read_pos_in = dac.readPosition;
+//Uint64 read_pos_float_in = dac.readPosition_float;
+//fprintf ( stderr , "gen_in read_pos=%d read_pos_f=%lx ratio=%lx\n" , read_pos_in,read_pos_float_in,crossbar.frequence_ratio );
 
 	if (crossbar.isDacMuted) {
 		/* Output sound = 0 */
 		for (i = 0; i < nSamplesToGenerate; i++) {
-			nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
-			MixBuffer[nBufIdx][0] = 0;
-			MixBuffer[nBufIdx][1] = 0;
+			nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
+			AudioMixBuffer[nBufIdx][0] = 0;
+			AudioMixBuffer[nBufIdx][1] = 0;
 		}
 
 		/* Counters are refreshed for when DAC becomes unmuted */
@@ -1866,7 +1888,7 @@ void Crossbar_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 	for (i = 0; i < nSamplesToGenerate; i++)
 	{
-		nBufIdx = (nMixBufIdx + i) % MIXBUFFER_SIZE;
+		nBufIdx = (nMixBufIdx + i) & AUDIOMIXBUFFER_SIZE_MASK;
 
 		/* ADC mixing (PSG sound or microphone sound for left and right channels) */
 		switch (crossbar.codecAdcInput) {
@@ -1879,17 +1901,17 @@ void Crossbar_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 			case 1:
 				/* Microphone sound for left channel, PSG sound for right channel */
 				adc_leftData = adc.buffer_left[crossbar.adc2dac_readBufferPosition];
-				adc_rightData = MixBuffer[nBufIdx][1];
+				adc_rightData = AudioMixBuffer[nBufIdx][1];
 				break;
 			case 2:
 				/* PSG sound for left channel, microphone sound for right channel */
-				adc_leftData = MixBuffer[nBufIdx][0];
+				adc_leftData = AudioMixBuffer[nBufIdx][0];
 				adc_rightData = adc.buffer_right[crossbar.adc2dac_readBufferPosition];
 				break;
 			case 3:
 				/* PSG sound for left and right channels */
-				adc_leftData = MixBuffer[nBufIdx][0];
-				adc_rightData = MixBuffer[nBufIdx][1];
+				adc_leftData = AudioMixBuffer[nBufIdx][0];
+				adc_rightData = AudioMixBuffer[nBufIdx][1];
 				break;
 		}
 
@@ -1931,8 +1953,8 @@ void Crossbar_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 				break;
 		}
 
-		MixBuffer[nBufIdx][0] = (dac_LeftData * crossbar.attenuationSettingLeft) >> 16;
-		MixBuffer[nBufIdx][1] = (dac_RightData * crossbar.attenuationSettingRight) >> 16;
+		AudioMixBuffer[nBufIdx][0] = (dac_LeftData * crossbar.attenuationSettingLeft) >> 16;
+		AudioMixBuffer[nBufIdx][1] = (dac_RightData * crossbar.attenuationSettingRight) >> 16;
 
 		/* Upgrade dac's buffer read pointer */
 		dac.readPosition_float += crossbar.frequence_ratio;
@@ -1950,6 +1972,8 @@ void Crossbar_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 
 		dac.readPosition = (dac.readPosition + n) % DACBUFFER_SIZE;
 		dac.readPosition_float &= 0xffffffff;			/* only keep the fractional part */
+//read_pos_float_in += crossbar.frequence_ratio;
+//fprintf ( stderr , "gen_one i=%d read_pos=%x read_pos_f=%lx ratio=%lx n=%d read_pos_f_total=%lx\n" , i, dac.readPosition,dac.readPosition_float,crossbar.frequence_ratio, n, read_pos_float_in );
 
 		/* Upgrade adc->dac's buffer read pointer */
 		crossbar.adc2dac_readBufferPosition_float += crossbar.frequence_ratio;
@@ -1958,6 +1982,8 @@ void Crossbar_GenerateSamples(int nMixBufIdx, int nSamplesToGenerate)
 		crossbar.adc2dac_readBufferPosition_float &= 0xffffffff;			/* only keep the fractional part */
 	}
 
+//fprintf ( stderr , "gen_out read_pos_delta=%x\n" , dac.readPosition-read_pos_in );
+
 	/* If the DAC didn't receive any data since last call to Crossbar_GenerateSamples() */
 	/* then we need to adjust dac.writePosition to be always ahead of dac.readPosition */
 	if ( dac.wordCount == 0 )
