@@ -107,6 +107,13 @@ struct videl_s {
 
 static struct videl_s videl;
 
+static void Videl_SetDefaultSavedRes(void)
+{
+	/* Default resolution to boot with */
+	videl.save_scrWidth = 640;
+	videl.save_scrHeight = 480;
+	videl.save_scrBpp = 4;
+}
 
 /**
  * Called upon startup (and via VIDEL_reset())
@@ -114,11 +121,7 @@ static struct videl_s videl;
 void Videl_Init(void)
 {
 	videl.hostColorsSync = false;
-
-	/* Default resolution to boot with */
-	videl.save_scrWidth = 640;
-	videl.save_scrHeight = 480;
-	videl.save_scrBpp = 4;
+	Videl_SetDefaultSavedRes();
 }
 
 /**
@@ -151,6 +154,12 @@ void VIDEL_MemorySnapShot_Capture(bool bSave)
 {
 	/* Save/Restore details */
 	MemorySnapShot_Store(&videl, sizeof(videl));
+
+	/* Make sure that the save_scr* variables match the ones during reset,
+	 * so that resolution changes get evaluated properly (e.g. to set the
+	 * right zooming variables */
+	if (!bSave)
+		Videl_SetDefaultSavedRes();
 }
 
 /**
@@ -592,27 +601,12 @@ void VIDEL_VMD_WriteWord(void)
 }
 
 
-/**
- *  VIDEL_getVideoramAddress: returns the video RAM address.
- *  On Falcon, video address must be a multiple of four in bitplane modes.
- */
-static Uint32 VIDEL_getVideoramAddress(void)
-{
-	Uint32 videoBase;
-
-	videoBase  = (Uint32) IoMem_ReadByte(0xff8201) << 16;
-	videoBase |= (Uint32) IoMem_ReadByte(0xff8203) << 8;
-	videoBase |= IoMem_ReadByte(0xff820d) & ~3;
-
-	return videoBase;
-}
-
 /**
  * Reset appropriate registers on VBL etc
  */
 void VIDEL_RestartVideoCounter(void)
 {
-	videl.videoRaster = VIDEL_getVideoramAddress();
+	videl.videoRaster = Video_GetScreenBaseAddr();
 	/* counter for VFC register $ff82a0 */
 	videl.vertFreqCounter = 0;
 }
@@ -941,7 +935,7 @@ bool VIDEL_renderScreen(void)
 
 	bool change = false;
 
-	Uint32 videoBase = VIDEL_getVideoramAddress();
+	Uint32 videoBase = Video_GetScreenBaseAddr();
 
 	if (vw > 0 && vw != videl.save_scrWidth) {
 		LOG_TRACE(TRACE_VIDEL, "Videl : width change from %d to %d\n", videl.save_scrWidth, vw);
@@ -990,7 +984,7 @@ bool VIDEL_renderScreen(void)
 
 	VIDEL_UpdateColors();
 
-	Screen_GenConvert(&STRam[videoBase], videl.XSize, videl.YSize,
+	Screen_GenConvert(videoBase, &STRam[videoBase], videl.XSize, videl.YSize,
 	                  videl.save_scrBpp, nextline, hscrolloffset,
 	                  videl.leftBorderSize, videl.rightBorderSize,
 	                  videl.upperBorderSize, videl.lowerBorderSize);
