@@ -60,7 +60,7 @@ CNF_PARAMS CurrentParams;
 
 
 // Keys to be listed in the Joysticks dropdowns
-SDLKey Preferences_KeysForJoysticks[] =
+SDL_Keycode Preferences_KeysForJoysticks[] =
 {
 	SDLK_BACKSPACE,
 	SDLK_TAB,
@@ -133,16 +133,16 @@ SDLKey Preferences_KeysForJoysticks[] =
 	SDLK_y,
 	SDLK_z,
 	SDLK_DELETE,
-	SDLK_KP0,
-	SDLK_KP1,
-	SDLK_KP2,
-	SDLK_KP3,
-	SDLK_KP4,
-	SDLK_KP5,
-	SDLK_KP6,
-	SDLK_KP7,
-	SDLK_KP8,
-	SDLK_KP9,
+	SDLK_KP_0,
+	SDLK_KP_1,
+	SDLK_KP_2,
+	SDLK_KP_3,
+	SDLK_KP_4,
+	SDLK_KP_5,
+	SDLK_KP_6,
+	SDLK_KP_7,
+	SDLK_KP_8,
+	SDLK_KP_9,
 	SDLK_KP_PERIOD,
 	SDLK_KP_DIVIDE,
 	SDLK_KP_MULTIPLY,
@@ -174,27 +174,20 @@ SDLKey Preferences_KeysForJoysticks[] =
 	SDLK_F13,
 	SDLK_F14,
 	SDLK_F15,
-	SDLK_NUMLOCK,
+	SDLK_NUMLOCKCLEAR,
 	SDLK_CAPSLOCK,
-	SDLK_SCROLLOCK,
+	SDLK_SCROLLLOCK,
 	SDLK_RSHIFT,
 	SDLK_LSHIFT,
 	SDLK_RCTRL,
 	SDLK_LCTRL,
 	SDLK_RALT,
 	SDLK_LALT,
-	SDLK_RMETA,
-	SDLK_LMETA,
-#if !WITH_SDL2
-	SDLK_LSUPER,
-	SDLK_RSUPER,
-	SDLK_COMPOSE,
-	SDLK_BREAK,
-	SDLK_EURO,
-#endif
+	SDLK_RGUI,
+	SDLK_LGUI,
 	SDLK_MODE,
 	SDLK_HELP,
-	SDLK_PRINT,
+	SDLK_PRINTSCREEN,
 	SDLK_SYSREQ,
 	SDLK_MENU,
 	SDLK_POWER,
@@ -558,7 +551,7 @@ BOOL flag1, flag2;
 	unsigned int i;
 	for (i = 0; i < Preferences_cKeysForJoysticks; i++)
 	{
-		SDLKey key = Preferences_KeysForJoysticks[i];
+		SDL_Keycode key = Preferences_KeysForJoysticks[i];
 		const char* szKeyName = SDL_GetKeyName(key);
 		[dropDown addItemWithTitle:[[NSString stringWithCString:szKeyName encoding:NSASCIIStringEncoding] capitalizedString]];
 		dropDown.lastItem.tag = key ;
@@ -577,13 +570,16 @@ BOOL flag1, flag2;
 	[midiInPort  addItemWithTitle:[NSString stringWithCString:szinPortName encoding:NSASCIIStringEncoding]];
 	[midiOutPort addItemWithTitle:[NSString stringWithCString:szinPortName encoding:NSASCIIStringEncoding]];
 	
+#ifdef HAVE_PORTMIDI
 	int i = 0;
-	const char* portName;
-	while ((portName = Midi_Host_GetPortName(i++, true)))
+	const char* portName = NULL;
+	while ((portName = Midi_Host_GetPortName(portName, +1, true)))
 		[midiInPort addItemWithTitle:[NSString stringWithCString:portName encoding:NSASCIIStringEncoding]];
 	i = 0;
-	while ((portName = Midi_Host_GetPortName(i++, false)))
+	portName = NULL;
+	while ((portName = Midi_Host_GetPortName(portName, +1, false)))
 		[midiOutPort addItemWithTitle:[NSString stringWithCString:portName encoding:NSASCIIStringEncoding]];
+#endif		// HAVE_PORTMIDI
 }
 
 // ----------------------------------------------------------------------------
@@ -600,10 +596,12 @@ BOOL flag1, flag2;
 //
 - (void)saveMidiDropdowns
 {
-	strncpy(ConfigureParams.Midi.sMidiInPortName,  [[midiInPort  titleOfSelectedItem] UTF8String], FILENAME_MAX);
-	strncpy(ConfigureParams.Midi.sMidiOutPortName, [[midiOutPort titleOfSelectedItem] UTF8String], FILENAME_MAX);
-	ConfigureParams.Midi.sMidiInPortName[FILENAME_MAX-1]  = 0;
-	ConfigureParams.Midi.sMidiOutPortName[FILENAME_MAX-1] = 0;
+	const char *midiOutPortUtf8String = [[midiOutPort titleOfSelectedItem] UTF8String];
+	const char *midiInPortUtf8String = [[midiInPort  titleOfSelectedItem] UTF8String];
+	strlcpy(ConfigureParams.Midi.sMidiOutPortName, midiOutPortUtf8String ? midiOutPortUtf8String : "Off",
+	        sizeof(ConfigureParams.Midi.sMidiOutPortName));
+	strlcpy(ConfigureParams.Midi.sMidiInPortName, midiInPortUtf8String ? midiInPortUtf8String : "Off",
+	        sizeof(ConfigureParams.Midi.sMidiInPortName));
 }
 
 
@@ -788,11 +786,9 @@ BOOL flag1, flag2;
 	IMPORT_SWITCH(FastBootPatch,ConfigureParams.System.bFastBoot);
 	IMPORT_RADIO(YMVoicesMixing,ConfigureParams.Sound.YmVolumeMixing);
 
-#if WITH_SDL2
 	IMPORT_SWITCH(SDL2UseGpuScaling, ConfigureParams.Screen.bUseSdlRenderer);
 	IMPORT_SWITCH(SDL2Resizable, ConfigureParams.Screen.bResizable);
 	IMPORT_SWITCH(SDL2UseVSync, ConfigureParams.Screen.bUseVsync);
-#endif
 
 	//deal with the Max Zoomed Stepper
 	IMPORT_NTEXTFIELD(maxZoomedWidth, ConfigureParams.Screen.nMaxWidth);
@@ -812,7 +808,6 @@ BOOL flag1, flag2;
 	}
 
 	//deal with TT RAM Size Stepper
-#ifdef ENABLE_WINUAE_CPU
 	int ttramsize_MB=ConfigureParams.Memory.TTRamSize_KB/1024 ;	//JV 12-2016
 
 	IMPORT_NTEXTFIELD(TTRAMSizeValue, ttramsize_MB); 			// MS 12-2016
@@ -831,8 +826,7 @@ BOOL flag1, flag2;
 		[FPUType selectCellWithTag:3];
 	//not needed anymore
 	//IMPORT_SWITCH(CompatibleFPU, ConfigureParams.System.bCompatibleFPU);
-#endif
-	
+
 	int i;
 	for (i = 0; i <= DLGSOUND_50KHZ-DLGSOUND_11KHZ; i++)
 	{
@@ -1050,21 +1044,19 @@ BOOL flag1, flag2;
 	// EXPORT_SWITCH(zoomSTLowRes, ConfigureParams.Screen.bZoomLowRes);
 	EXPORT_SWITCH(showStatusBar,ConfigureParams.Screen.bShowStatusbar);
 	EXPORT_DROPDOWN(enableDSP,ConfigureParams.System.nDSPType);
-	
+
 	EXPORT_SWITCH(falconTTRatio, ConfigureParams.Screen.bAspectCorrect);
 	EXPORT_SWITCH(fullScreen, ConfigureParams.Screen.bFullScreen);
 	EXPORT_SWITCH(ledDisks, ConfigureParams.Screen.bShowDriveLed);
 	EXPORT_SWITCH(keepDesktopResolution, ConfigureParams.Screen.bKeepResolution);
-	
+
 	//v1.6.1
 	EXPORT_SWITCH(FastBootPatch,ConfigureParams.System.bFastBoot);
 	EXPORT_RADIO(YMVoicesMixing,ConfigureParams.Sound.YmVolumeMixing);
 
-#if WITH_SDL2
 	EXPORT_SWITCH(SDL2UseGpuScaling, ConfigureParams.Screen.bUseSdlRenderer);
 	EXPORT_SWITCH(SDL2Resizable, ConfigureParams.Screen.bResizable);
 	EXPORT_SWITCH(SDL2UseVSync, ConfigureParams.Screen.bUseVsync);
-#endif
 
 	EXPORT_NTEXTFIELD(maxZoomedWidth, ConfigureParams.Screen.nMaxWidth);
 	EXPORT_NTEXTFIELD(maxZoomedHeight, ConfigureParams.Screen.nMaxHeight);
@@ -1078,9 +1070,8 @@ BOOL flag1, flag2;
 	 case 3: ConfigureParams.System.VideoTimingMode=VIDEO_TIMING_MODE_WS3; break;
 	 case 4: ConfigureParams.System.VideoTimingMode=VIDEO_TIMING_MODE_WS4; break;
 	}
-#ifdef ENABLE_WINUAE_CPU
-	int ttramsizeMB=[TTRAMSizeValue intValue]*1024;										//JV 12-2016
-	ConfigureParams.Memory.TTRamSize_KB=ttramsizeMB;
+	int ttramsize_MB=[TTRAMSizeValue intValue];							//JV 12-2016
+	ConfigureParams.Memory.TTRamSize_KB=1024*ttramsize_MB;
 
 	//EXPORT_NTEXTFIELD(TTRAMSizeValue, ConfigureParams.Memory.TTRamSize_KB);			// MS 12-2016
 
@@ -1097,7 +1088,6 @@ BOOL flag1, flag2;
 		ConfigureParams.System.n_FPUType = FPU_CPU;
 	//not needed anymore
 	//EXPORT_SWITCH(CompatibleFPU, ConfigureParams.System.bCompatibleFPU);
-#endif
 
 	ConfigureParams.Sound.nPlaybackFreq = nSoundFreqs[[[playbackQuality selectedCell] tag]];
 
@@ -1221,7 +1211,6 @@ BOOL flag1, flag2;
 	configNm = [NSMutableString stringWithCapacity:50] ; [configNm setString:@""] ; [configNm retain] ;
 	opnPanel = [NSOpenPanel openPanel]; [opnPanel retain] ;
 	savPanel = [NSSavePanel savePanel]; [savPanel retain] ;
-#ifdef ENABLE_WINUAE_CPU
 	cycleExactCPU.enabled = true;
 	MMU_Emulation.enabled = true;
 	adressSpace24.enabled = true;
@@ -1229,8 +1218,6 @@ BOOL flag1, flag2;
 	//CompatibleFPU.enabled = true;
 	FPUType.enabled = true;
 	bCell68060.enabled = true;
-#endif
-    
 }
 
 @end
