@@ -55,8 +55,8 @@ const char HDC_fileid[] = "Hatari hdc.c";
 
 /* HDC globals */
 static SCSI_CTRLR AcsiBus;
-int nAcsiPartitions = 0;
-bool bAcsiEmuOn = false;
+int nAcsiPartitions;
+bool bAcsiEmuOn;
 
 /* Our dummy INQUIRY response data */
 static unsigned char inquiry_bytes[] =
@@ -97,6 +97,31 @@ static unsigned long HDC_GetLBA(SCSI_CTRLR *ctr)
 		return HDC_ReadInt32(ctr->command, 2);	/* Class 1 */
 }
 
+/**
+ * Return number of bytes for a command block.
+ */
+static int HDC_GetCommandByteCount(SCSI_CTRLR *ctr)
+{
+	if (ctr->opcode == 0x88 || ctr->opcode == 0x8a || ctr->opcode == 0x8f ||
+	    ctr->opcode == 0x91 || ctr->opcode == 0x9e || ctr->opcode == 0x9f)
+	{
+		return 16;
+	}
+	else if (ctr->opcode == HD_REPORT_LUNS)
+	{
+		return 12;
+	}
+	else if (ctr->opcode == 0x05 || (ctr->opcode >= 0x20 && ctr->opcode <= 0x7d))
+	{
+		return 10;
+	}
+	else
+	{
+		return 6;
+	}
+}
+
+
 /**
  * Return the count specified in the current ACSI command block.
  */
@@ -108,17 +133,6 @@ static int HDC_GetCount(SCSI_CTRLR *ctr)
 		return HDC_ReadInt16(ctr->command, 7);	/* Class 1 */
 }
 
-/**
- * Return the control byte specified in the current ACSI command block.
- */
-static inline Uint8 HDC_GetControl(SCSI_CTRLR *ctr)
-{
-	if (ctr->opcode < 0x20)
-		return ctr->command[5];			/* Class 0 */
-	else
-		return ctr->command[9];			/* Class 1 */
-}
-
 /**
  * Get pointer to response buffer, set up size indicator - and allocate
  * a new buffer if it is not big enough
@@ -142,12 +156,19 @@ static Uint8 *HDC_PrepRespBuf(SCSI_CTRLR *ctr, int size)
  */
 static inline char *HDC_CmdInfoStr(SCSI_CTRLR *ctr)
 {
-	static char str[80];
+	char cdb[80] = { 0 };
 
-	snprintf(str, sizeof(str),
-	         "%s, t=%i, lun=%i, opc=0x%x, cnt=0x%x, ctrl=0x%x",
-	         ctr->typestr, ctr->target, HDC_GetLUN(ctr), ctr->opcode,
-	         HDC_GetCount(ctr), HDC_GetControl(ctr));
+	for (int i = 0; i < HDC_GetCommandByteCount(ctr); i++)
+	{
+		char tmp[5];
+		snprintf(tmp, sizeof(tmp), "%s%02x", i ? ":" : "", ctr->command[i]);
+		strcat(cdb, tmp);
+	}
+
+	static char str[160];
+
+	snprintf(str, sizeof(str), "%s, t=%i, lun=%i, cdb=%s",
+	         ctr->typestr, ctr->target, HDC_GetLUN(ctr), cdb);
 
 	return str;
 }
@@ -300,10 +321,8 @@ static void HDC_Cmd_RequestSense(SCSI_CTRLR *ctr)
  * Mode sense - Vendor specific page 00h.
  * (Just enough to make the HDX tool from AHDI 5.0 happy)
  */
-static void HDC_CmdModeSense0x00(SCSI_DEV *dev, SCSI_CTRLR *ctr)
+static void HDC_CmdModeSense0x00(SCSI_DEV *dev, SCSI_CTRLR *ctr, Uint8 *buf)
 {
-	Uint8 *buf = HDC_PrepRespBuf(ctr, 16);
-
 	buf[0] = 0;
 	buf[1] = 14;
 	buf[2] = 0;
@@ -330,10 +349,8 @@ static void HDC_CmdModeSense0x00(SCSI_DEV *dev, SCSI_CTRLR *ctr)
 /**
  * Mode sense - Rigid disk geometry page (requested by ASV).
  */
-static void HDC_CmdModeSense0x04(SCSI_DEV *dev, SCSI_CTRLR *ctr)
+static void HDC_CmdModeSense0x04(SCSI_DEV *dev, SCSI_CTRLR *ctr, Uint8 *buf)
 {
-	Uint8 *buf = HDC_PrepRespBuf(ctr, 24);
-
 	buf[0] = 4;
 	buf[1] = 22;
 
@@ -364,8 +381,8 @@ static void HDC_CmdModeSense0x04(SCSI_DEV *dev, SCSI_CTRLR *ctr)
 
 	buf[19] = 0;
 
-	buf[20] = 0;
-	buf[21] = 0;
+	buf[20] = 0x1c;	// Medium rotation rate 7200
+	buf[21] = 0x20;
 
 	buf[22] = 0;
 	buf[23] = 0;
@@ -377,6 +394,7 @@ static void HDC_CmdModeSense0x04(SCSI_DEV *dev, SCSI_CTRLR *ctr)
  */
 static void HDC_Cmd_ModeSense(SCSI_CTRLR *ctr)
 {
+	Uint8 *buf;
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
 
 	LOG_TRACE(TRACE_SCSI_CMD, "HDC: MODE SENSE (%s).\n", HDC_CmdInfoStr(ctr));
@@ -386,15 +404,31 @@ static void HDC_Cmd_ModeSense(SCSI_CTRLR *ctr)
 	switch(ctr->command[2])
 	{
 	 case 0x00:
-		HDC_CmdModeSense0x00(dev, ctr);
+		buf = HDC_PrepRespBuf(ctr, 16);
+		HDC_CmdModeSense0x00(dev, ctr, buf);
 		break;
 
 	 case 0x04:
-		HDC_CmdModeSense0x04(dev, ctr);
+		buf = HDC_PrepRespBuf(ctr, 28);
+		HDC_CmdModeSense0x04(dev, ctr, buf + 4);
+		buf[0] = 24;
+		buf[1] = 0;
+		buf[2] = 0;
+		buf[3] = 0;
+		break;
+
+	 case 0x3f:
+		buf = HDC_PrepRespBuf(ctr, 44);
+		HDC_CmdModeSense0x04(dev, ctr, buf + 4);
+		HDC_CmdModeSense0x00(dev, ctr, buf + 28);
+		buf[0] = 44;
+		buf[1] = 0;
+		buf[2] = 0;
+		buf[3] = 0;
 		break;
 
 	 default:
-		Log_Printf(LOG_TODO, "HDC: Unsupported MODE SENSE command\n");
+		Log_Printf(LOG_TODO, "HDC: Unsupported MODE SENSE mode page\n");
 		ctr->status = HD_STATUS_ERROR;
 		dev->nLastError = HD_REQSENS_INVARG;
 		return;
@@ -422,6 +456,31 @@ static void HDC_Cmd_FormatDrive(SCSI_CTRLR *ctr)
 }
 
 
+/**
+ * Report LUNs.
+ */
+static void HDC_Cmd_ReportLuns(SCSI_CTRLR *ctr)
+{
+	SCSI_DEV *dev = &ctr->devs[ctr->target];
+	Uint8 *buf;
+
+	LOG_TRACE(TRACE_SCSI_CMD, "HDC: REPORT LUNS (%s).\n", HDC_CmdInfoStr(ctr));
+
+	buf = HDC_PrepRespBuf(ctr, 16);
+
+	/* LUN list length, 8 bytes per LUN */
+	buf[0] = 0;
+	buf[1] = 0;
+	buf[2] = 0;
+	buf[3] = 8;
+	memset(&buf[4], 0, 12);
+
+	ctr->status = HD_STATUS_OK;
+	dev->nLastError = HD_REQSENS_OK;
+	dev->bSetLastBlockAddr = false;
+}
+
+
 /**
  * Read capacity of our disk.
  */
@@ -608,6 +667,10 @@ static void HDC_EmulateCommandPacket(SCSI_CTRLR *ctr)
 		HDC_Cmd_FormatDrive(ctr);
 		break;
 
+	case HD_REPORT_LUNS:
+		HDC_Cmd_ReportLuns(ctr);
+		break;
+
 	 /* as of yet unsupported commands */
 	 case HD_VERIFY_TRACK:
 	 case HD_FORMAT_TRACK:
@@ -715,8 +778,8 @@ int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel, int *pIsByteSwapped)
 		{
 			boot = pinfo[0];
 			ptype = pinfo[4];
-			start = SDL_SwapLE32(*(long*)(pinfo+8));
-			sectors = SDL_SwapLE32(*(long*)(pinfo+12));
+			start = SDL_SwapLE32(*(Uint32*)(pinfo+8));
+			sectors = SDL_SwapLE32(*(Uint32*)(pinfo+12));
 			total += sectors;
 			LOG_TRACE(tracelevel, "- Partition %d: type=0x%02x, start=0x%08x, size=%.1f MB %s%s\n",
 				  i, ptype, start, sectors/2048.0, boot ? "(boot)" : "", sectors ? "" : "(invalid)");
@@ -958,9 +1021,10 @@ bool HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 		ctr->command[ctr->byteCount] = b;
 	++ctr->byteCount;
 
-	/* have we received a complete 6-byte class 0 or 10-byte class 1 packet yet? */
+	/* have we received a complete command packet yet? */
 	if ((ctr->opcode < 0x20 && ctr->byteCount == 6) ||
-	    (ctr->opcode >= 0x20 && ctr->opcode < 0x60 && ctr->byteCount == 10))
+	    (ctr->opcode >= 0x20 && ctr->opcode < 0x60 && ctr->byteCount == 10) ||
+	    (ctr->opcode == HD_REPORT_LUNS && ctr->byteCount == 12))
 	{
 		/* We currently only support LUN 0, however INQUIRY must
 		 * always be handled, see SCSI standard */
@@ -986,7 +1050,7 @@ bool HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 			}
 		}
 	}
-	else if (ctr->opcode >= 0x60)
+	else if (ctr->opcode >= 0x60 && ctr->opcode != HD_REPORT_LUNS)
 	{
 		/* Commands >= 0x60 are not supported right now */
 		ctr->status = HD_STATUS_ERROR;
