@@ -102,6 +102,9 @@ typedef struct
 {
   bool bDisableKeyRepeat;
   KEYMAPTYPE nKeymapType;
+  int nCountryCode;
+  int nKbdLayout;
+  int nLanguage;
   char szMappingFileName[FILENAME_MAX];
 } CNF_KEYBOARD;
 
@@ -298,14 +301,10 @@ typedef struct
   bool bForceMax;
   bool bUseExtVdiResolutions;
   bool bKeepResolution;
-#if !WITH_SDL2
-  bool bKeepResolutionST;
-#else
   bool bResizable;
   bool bUseVsync;
   bool bUseSdlRenderer;
   float nZoomFactor;
-#endif
   int nSpec512Threshold;
   int nForceBpp;
   int nVdiColors;
@@ -324,6 +323,7 @@ typedef struct
   char szPrintToFileName[FILENAME_MAX];
 } CNF_PRINTER;
 
+#define MAX_MIDI_PORT_NAME 256 /* a guess */
 
 /* Midi configuration */
 typedef struct
@@ -331,8 +331,8 @@ typedef struct
   bool bEnableMidi;
   char sMidiInFileName[FILENAME_MAX];
   char sMidiOutFileName[FILENAME_MAX];
-  char sMidiInPortName[FILENAME_MAX];
-  char sMidiOutPortName[FILENAME_MAX];
+  char sMidiInPortName[MAX_MIDI_PORT_NAME];
+  char sMidiOutPortName[MAX_MIDI_PORT_NAME];
 } CNF_MIDI;
 
 
@@ -354,7 +354,12 @@ typedef enum
   DSP_TYPE_EMU
 } DSPTYPE;
 
-#if ENABLE_WINUAE_CPU
+typedef enum
+{
+  VME_TYPE_NONE,
+  VME_TYPE_DUMMY
+} VMETYPE;
+
 typedef enum
 {
   FPU_NONE = 0,
@@ -362,7 +367,6 @@ typedef enum
   FPU_68882 = 68882,
   FPU_CPU = 68040
 } FPUTYPE;
-#endif
 
 typedef enum
 {
@@ -381,19 +385,18 @@ typedef struct
   MACHINETYPE nMachineType;
   bool bBlitter;                  /* TRUE if Blitter is enabled */
   DSPTYPE nDSPType;               /* how to "emulate" DSP */
+  VMETYPE nVMEType;               /* how to "emulate" SCU/VME */
   bool bPatchTimerD;
   bool bFastBoot;                 /* Enable to patch TOS for fast boot */
   bool bFastForward;
-  bool bAddressSpace24;           /* Always set to true with old UAE cpu */
+  bool bAddressSpace24;           /* true if using a 24-bit address bus */
   VIDEOTIMINGMODE VideoTimingMode;
 
-#if ENABLE_WINUAE_CPU
   bool bCycleExactCpu;
   FPUTYPE n_FPUType;
   bool bCompatibleFPU;            /* More compatible FPU */
   bool bSoftFloatFPU;
   bool bMMU;                      /* TRUE if MMU is enabled */
-#endif
 } CNF_SYSTEM;
 
 typedef struct
