@@ -26,8 +26,7 @@
 #include "cycInt.h"
 #include "log.h"
 
-
-/* 68000 Register defines */
+/* 68000 register defines */
 enum {
   REG_D0,    /* D0.. */
   REG_D1,
@@ -132,7 +131,9 @@ enum {
 /* Bus access mode */
 #define	BUS_MODE_CPU		0			/* bus is owned by the cpu */
 #define	BUS_MODE_BLITTER	1			/* bus is owned by the blitter */
-
+#define BUS_MODE_DEBUGGER	2			/* bus is owned by debugger : special case to access RAM 0-0x7FF */
+							/* without doing a bus error when not in supervisor mode */
+							/* eg : this is used in some debug functions that do get_long/word/byte */
 
 /* [NP] Notes on IACK :
  * When an interrupt happens, it's possible a similar interrupt happens again
@@ -188,7 +189,7 @@ enum {
 #define CPU_IACK_CYCLES_MFP_CE		12		/* vector sent by the MFP (TODO value not measured on real STF) */
 #define CPU_IACK_CYCLES_VIDEO_CE	10		/* auto vectored for HBL/VBL (value measured on real STF) */
 
-/* Values for IACK sequence when running in normal/prefetch mode or when using old UAE CPU */
+/* Values for IACK sequence when running in normal/prefetch mode */
 #define CPU_IACK_CYCLES_START		12		/* number of cycles before starting the IACK when not using CE mode */
 							/* (this should be a multiple of 4, else it will be rounded by M68000_AddCycles) */
 #define CPU_IACK_CYCLES_MFP		12		/* vector sent by the MFP */
@@ -213,6 +214,7 @@ extern int nCpuFreqShift;
 extern int WaitStateCycles;
 extern int BusMode;
 extern bool	CPU_IACK;
+extern bool	CpuRunCycleExact;
 
 extern int	LastOpcodeFamily;
 extern int	LastInstrCycles;
@@ -229,7 +231,9 @@ extern const char *OpcodeName[];
 static inline void M68000_AddCycles(int cycles)
 {
 	cycles = (cycles + 3) & ~3;
+#ifndef CYCINT_NEW
 	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
+#endif
 	nCyclesMainCounter += cycles;
 	CyclesGlobalClockCounter += cycles;
 }
@@ -320,8 +324,9 @@ static inline void M68000_AddCyclesWithPairing(int cycles)
 		cycles = (cycles + 3) & ~3;		/* no pairing, round current instr to 4 cycles */
 	}
 
+#ifndef CYCINT_NEW
 	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-
+#endif
 	nCyclesMainCounter += cycles;
 	CyclesGlobalClockCounter += cycles;
 	BusCyclePenalty = 0;
@@ -340,8 +345,9 @@ static inline void M68000_AddCyclesWithPairing(int cycles)
  */
 static inline void M68000_AddCycles_CE(int cycles)
 {
+#ifndef CYCINT_NEW
 	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-
+#endif
 	nCyclesMainCounter += cycles;
 	CyclesGlobalClockCounter += cycles;
 }
@@ -373,5 +379,6 @@ extern void M68000_ChangeCpuFreq ( void );
 extern Uint16 M68000_GetSR ( void );
 extern void M68000_SetSR ( Uint16 v );
 extern void M68000_SetPC ( uaecptr v );
+extern void M68000_MMU_Info(FILE *fp, Uint32 flags);
 
 #endif
