@@ -11,11 +11,7 @@ extern bool bLoadAutoSave;
 extern bool bLoadMemorySave;
 extern bool AviRecordOnStartup;
 extern bool BenchmarkMode;
-extern int ConOutDevice;
 
-#define CONOUT_DEVICE_NONE 127 /* valid ones are 0-7 */
-
-extern Uint32 Opt_GetNoParachuteFlag(void);
 extern bool Opt_IsAtariProgram(const char *path);
 extern bool Opt_ShowError(unsigned int optid, const char *value, const char *error);
 extern int Opt_ValueAlignMinMax(int value, int align, int min, int max);
