@@ -6,11 +6,12 @@
 
   TOS *.INF file overloading for autostarting & TOS resolution overriding.
 */
-const char INFFILE_fileid[] = "Hatari inffile.c : " __DATE__ " " __TIME__;
+const char INFFILE_fileid[] = "Hatari inffile.c";
 
 #include <SDL_endian.h>
 #include <assert.h>
 #include <errno.h>
+#include <ctype.h>
 
 #include "main.h"
 #include "configuration.h"
@@ -20,6 +21,7 @@ const char INFFILE_fileid[] = "Hatari inffile.c : " __DATE__ " " __TIME__;
 #include "file.h"
 #include "log.h"
 #include "str.h"
+#include "screen.h"
 #include "tos.h"
 #include "vdi.h"
 
@@ -41,6 +43,9 @@ typedef enum {
 	RES_COUNT
 } res_value_t;
 
+/* map values 0-6 to EmuTOS: N/A, ST low, med, high, TT med, high, low */
+static const res_value_t emutos_map[] = { 0, 0, 1, 2, 4, 6, 7 };
+
 
 static struct {
 	FILE *file;          /* file pointer to contents of INF file */
@@ -58,10 +63,15 @@ static struct {
  * '@' character in the INF files #Z line
  * (first value is 00: TOS, 01: GEM).
  *
- * Resolution is specified in the 2nd hex value in #E line
- * in normal TOS and EmuTOS <= 0.9.6, and in 4th hex value
- * in EmuTOS >= 0.9.7.  Hatari supports only latter EmuTOS
- * versions.
+ * #E line content differs between TOS versions:
+ * + Atari TOS:
+ *   - Resolution is specified in the 2nd hex value
+ *   - Blitter enabling is 0x10 bit for that
+ * + EmuTOS v0.9.7 or newer:
+ *   - Resolution is specified in the 4th hex value
+ *   - Blitter enabling is 0x80 bit in the 2nd hex value
+ * + Older EmuTOS versions (not supported!):
+ *   - Resolution is in the 2nd hex value
  *
  * TOS versions expect both of these to be within certain
  * number of bytes from the beginning of the file, and there
@@ -69,12 +79,15 @@ static struct {
  *
  * More documentation on the DESKTOP.INF file content:
  * http://st-news.com/issues/st-news-volume-2-issue-6/education/the-desktopinf-file/
+ *
+ * EmuTOS INF file content is documented only in the sources:
+ * https://github.com/emutos/emutos/blob/master/desk/deskapp.c
  */
 
 /* EmuDesk INF file format and values differ from normal TOS */
 static const char emudesk_inf[] =
 "#R 01\r\n"
-"#E 1A 61 FF 00 00\r\n"
+"#E 1A E1 FF 00 00\r\n"
 "#W 00 00 02 08 26 0C 00 @\r\n"
 "#W 00 00 02 0A 26 0C 00 @\r\n"
 "#W 00 00 02 0D 26 0C 00 @\r\n"
@@ -176,7 +189,7 @@ bool INF_SetAutoStart(const char *name, int opt_id)
 		/* full path */
 		const char *ptr;
 		int offset;
-		prgname = malloc(len+2);
+		prgname = Str_Alloc(len + 1);  /* +1 for additional backslash */
 		ptr = strrchr(name, '\\');
 		if (ptr)
 			offset = ptr - name + 1;
@@ -192,6 +205,7 @@ bool INF_SetAutoStart(const char *name, int opt_id)
 			/* NOT OK: A:DIR\NAME.PRG */
 			if (ptr)
 			{
+				Log_Printf(LOG_WARN, "rejecting auto-start path that doesn't have '\\' after drive ID:\n\t%s\n", name);
 				free(prgname);
 				return false;
 			}
@@ -207,12 +221,13 @@ bool INF_SetAutoStart(const char *name, int opt_id)
 	else if (strchr(name, '\\'))
 	{
 		/* partial path not accepted */
+		Log_Printf(LOG_WARN, "rejecting auto-start path starting with '\\', but without drive ID:\n\t%s\n", name);
 		return false;
 	}
 	else
 	{
 		/* just program -> add path */
-		prgname = malloc(3 + len + 1);
+		prgname = Str_Alloc(3 + len);
 		strcpy(prgname, "C:\\");
 		Str_Filename2TOSname(name, prgname+3);
 	}
@@ -337,12 +352,35 @@ int INF_ValidateAutoStart(const char **val, const char **err)
 
 
 /**
- * Map / set VDI to INF file resolution
+ * Map VDI / HW resolution to INF file resolution value
  */
 static res_value_t vdi2inf(res_value_t mode)
 {
-	res_value_t res = TosOverride.reso;
-	res_value_t newres = mode + 1;
+	res_value_t newres, res = TosOverride.reso;
+
+	switch (mode)
+	{
+	case ST_LOW_RES:
+		newres = RES_ST_LOW;
+		break;
+	case ST_MEDIUM_RES:
+		newres = RES_ST_MED;
+		break;
+	case ST_HIGH_RES:
+		newres = RES_ST_HIGH;
+		break;
+	case TT_LOW_RES:
+		newres = RES_TT_LOW;
+		break;
+	case TT_MEDIUM_RES:
+		newres = RES_TT_MED;
+		break;
+	case TT_HIGH_RES:
+		newres = RES_TT_HIGH;
+		break;
+	default:
+		newres = res;
+	}
 	if (res != newres)
 	{
 		if (res)
@@ -466,9 +504,7 @@ static int INF_ValidateResolution(int *set_res, const char **val, const char **e
 
 	if (bIsEmuTOS)
 	{
-		/* map values 0-6: N/A, ST low, med, high, TT med, high, low */
-		unsigned char map[] = { 0, 0, 1, 2, 4, 6, 7 };
-		res = map[res];
+		res = emutos_map[res];
 		Log_Printf(LOG_DEBUG, "Remapped INF file TOS resolution for EmuTOS\n");
 	}
 	else if (TosVersion >= 0x0160)
@@ -514,8 +550,7 @@ static char *get_builtin_inf(const char *contents)
 
 	inflen = strlen(contents);
 	winlen = strlen(drivewin);
-	inf = malloc(inflen + winlen + 1);
-	assert(inf);
+	inf = Str_Alloc(inflen + winlen);
 
 	/* drive letter offset on drive window line */
 	driveoffset = strchr(drivewin, 'X') - drivewin;
@@ -582,7 +617,7 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 		*res_col = 12;
 	}
 	/* need to match file TOS searches first */
-	else if (TosVersion >= 0x0200)
+	else if (TosVersion >= 0x0200 && TosVersion != 0x300)
 	{
 		infname = "NEWDESK.INF";
 		size = sizeof(newdesk_inf);
@@ -606,8 +641,7 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 		return get_builtin_inf(contents);
 	}
 
-	hostname = malloc(FILENAME_MAX);
-	assert(hostname);
+	hostname = Str_Alloc(FILENAME_MAX);
 
 	/* convert to host file name, and read that */
 	GemDOS_CreateHardDriveFileName(2, infname, hostname, FILENAME_MAX);
@@ -615,7 +649,7 @@ static char *get_inf_file(const char **set_infname, int *set_size, int *res_col)
 	GemDOS_Info(stderr, 0);
 	fprintf(stderr, "\nChecking for existing INF file '%s' -> '%s'...\n", infname, hostname);
 #endif
-	host_content = File_Read(hostname, &host_size, NULL);
+	host_content = File_ReadAsIs(hostname, &host_size);
 
 	if (host_content)
 	{
@@ -698,6 +732,7 @@ static FILE* write_inf_file(const char *contents, int size, int res, int res_col
 #else
 # if INF_DEBUG
 	{
+		/* insecure file path + leaving it behind for debugging */
 		const char *debugfile = "/tmp/hatari-desktop-inf.txt";
 		fprintf(stderr, "Virtual INF file: '%s'\n", debugfile);
 		fp = fopen(debugfile, "w+b");
