@@ -23,9 +23,6 @@
 
 bool bUseLilo;
 
-/* needs WinUAE CPU for MMU etc support */
-#if ENABLE_WINUAE_CPU
-
 #define LILO_DEBUG 1
 #if LILO_DEBUG
 #define Dprintf(a) printf a
@@ -272,7 +269,7 @@ static bool add_bi_string(uint16_t tag, const char *s);
 
 bool lilo_init(void)
 {
-	uint8_t *ROMBaseHost = STRam + TosAddress;
+	uint8_t *ROMBaseHost = &RomMem[TosAddress];
 
 	if (!ConfigureParams.System.bMMU || ConfigureParams.Memory.STRamSize_KB < 8*1024) {
 		Log_AlertDlg(LOG_FATAL, "Linux requires MMU and at least 8MB of RAM!");
@@ -284,7 +281,7 @@ bool lilo_init(void)
 	ROMBaseHost[0x0002] = 0x4e;		/* jmp <abs.addr> */
 	ROMBaseHost[0x0003] = 0xf9;
 
-	/* TODO: 0x30 is AB40 reset address, not Falcon/TT one */
+	/* TODO: ROM + 0x30 is Linux reset address on AB40, 0x4 on Falcon/TT */
 #if 0
 	if (!(ConfigureParams.Log.bNatFeats && ConfigureParams.Lilo.bHaltOnReboot)) {
 		/* set up a minimal OS for successful Linux/m68k reboot */
@@ -675,7 +672,7 @@ static bool check_kernel(void *kernel, Elf32_Addr *kernel_offset,
 	tmp = (uint32_t *)RAMBaseHost;
 	tmp[0] = SDL_SwapBE32(*kernel_offset + KERNEL_START);	/* SP */
 	tmp[1] = SDL_SwapBE32(TosAddress);		/* PC = ROMBase */
-	uint8_t *ROMBaseHost = STRam + TosAddress;
+	uint8_t *ROMBaseHost = &RomMem[TosAddress];
 	ROMBaseHost[4] = (*kernel_offset + KERNEL_START) >> 24;
 	ROMBaseHost[5] = (*kernel_offset + KERNEL_START) >> 16;
 	ROMBaseHost[6] = (*kernel_offset + KERNEL_START) >>  8;
@@ -727,7 +724,7 @@ static bool set_machine_type(void)
 		 */
 		if (ConfigureParams.System.nMachineType == MACHINE_FALCON) {
 			/* let's try claiming it's Falcon AfterBurner like Aranym does */
-			bi.mch_cookie = SDL_SwapBE32(ATARI_MCH_AB40);
+			bi.mch_type = SDL_SwapBE32(ATARI_MACH_AB40);
 		}
 #endif
 		break;
@@ -845,13 +842,3 @@ static bool add_bi_string(uint16_t tag, const char *s)
 {
 	return add_bi_record(tag, strlen(s) + 1, s);
 }
-
-#else
-
-bool lilo_init(void)
-{
-	/* never called as nothing sets bUseLilo true */
-	return false;
-}
-
-#endif /* ENABLE_WINUAE_CPU */
