@@ -206,7 +206,7 @@ bool Main_PauseEmulation(bool visualize)
 
 		if (bGrabMouse && !bInFullScreen)
 			/* Un-grab mouse pointer in windowed mode */
-			SDL_WM_GrabInput(SDL_GRAB_OFF);
+			SDL_SetRelativeMouseMode(false);
 	}
 	return true;
 }
@@ -231,7 +231,7 @@ bool Main_UnPauseEmulation(void)
 
 	if (bGrabMouse)
 		/* Grab mouse pointer again */
-		SDL_WM_GrabInput(SDL_GRAB_ON);
+		SDL_SetRelativeMouseMode(true);
 	return true;
 }
 
@@ -451,11 +451,8 @@ void Main_WarpMouse(int x, int y, bool restore)
 		return;
 	if (!bAllowMouseWarp)
 		return;
-#if WITH_SDL2
+
 	SDL_WarpMouseInWindow(sdlWindow, x, y);
-#else
-	SDL_WarpMouse(x, y);
-#endif
 	bIgnoreNextMouseMotion = true;
 }
 
@@ -549,12 +546,10 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_KEYDOWN:
-#if WITH_SDL2
 			if (event.key.repeat) {
 				bContinueProcessing = true;
 				break;
 			}
-#endif
 			Keymap_KeyDown(&event.key.keysym);
 			break;
 
@@ -582,18 +577,6 @@ void Main_EventHandler(void)
 				/* Start double-click sequence in emulation time */
 				Keyboard.LButtonDblClk = 1;
 			}
-#if !WITH_SDL2
-			else if (event.button.button == SDL_BUTTON_WHEELDOWN)
-			{
-				/* Simulate pressing the "cursor down" key */
-				IKBD_PressSTKey(0x50, true);
-			}
-			else if (event.button.button == SDL_BUTTON_WHEELUP)
-			{
-				/* Simulate pressing the "cursor up" key */
-				IKBD_PressSTKey(0x48, true);
-			}
-#endif
 			break;
 
 		 case SDL_MOUSEBUTTONUP:
@@ -605,21 +588,8 @@ void Main_EventHandler(void)
 			{
 				Keyboard.bRButtonDown &= ~BUTTON_MOUSE;
 			}
-#if !WITH_SDL2
-			else if (event.button.button == SDL_BUTTON_WHEELDOWN)
-			{
-				/* Simulate releasing the "cursor down" key */
-				IKBD_PressSTKey(0x50, false);
-			}
-			else if (event.button.button == SDL_BUTTON_WHEELUP)
-			{
-				/* Simulate releasing the "cursor up" key */
-				IKBD_PressSTKey(0x48, false);
-			}
-#endif
 			break;
 
-#if WITH_SDL2
 		 case SDL_MOUSEWHEEL:
 			/* Simulate cursor keys on mouse wheel events */
 			if (event.wheel.x > 0)
@@ -680,15 +650,6 @@ void Main_EventHandler(void)
 				bAllowMouseWarp = false;
 				break;
 			}
-#else /* !WITH_SDL2 */
-		case SDL_ACTIVEEVENT:
-			Log_Printf(LOG_DEBUG, "SDL1 active event 0x%x = %d\n",
-				   event.active.state, event.active.gain);
-			if (event.active.gain)
-				bAllowMouseWarp = true;
-			else
-				bAllowMouseWarp = false;
-#endif /* !WITH_SDL2 */
 			bContinueProcessing = true;
 			break;
 
@@ -707,17 +668,10 @@ void Main_EventHandler(void)
  */
 void Main_SetTitle(const char *title)
 {
-#if WITH_SDL2
 	if (title)
 		SDL_SetWindowTitle(sdlWindow, title);
 	else
 		SDL_SetWindowTitle(sdlWindow, PROG_NAME);
-#else
-	if (title)
-		SDL_WM_SetCaption(title, "Hatari");
-	else
-		SDL_WM_SetCaption(PROG_NAME, "Hatari");
-#endif
 }
 
 /*-----------------------------------------------------------------------*/
@@ -751,7 +705,7 @@ static void Main_Init(void)
 
 	/* Init SDL's video subsystem. Note: Audio subsystem
 	   will be initialized later (failure not fatal). */
-	if (SDL_Init(SDL_INIT_VIDEO | Opt_GetNoParachuteFlag()) < 0)
+	if (SDL_Init(SDL_INIT_VIDEO) < 0)
 	{
 		fprintf(stderr, "ERROR: could not initialize the SDL library:\n %s\n", SDL_GetError() );
 		exit(-1);
@@ -872,8 +826,11 @@ static void Main_LoadInitialConfig(void)
 {
 	char *psGlobalConfig;
 
-	psGlobalConfig = malloc(FILENAME_MAX);
-	if (psGlobalConfig && !getenv("HATARI_TEST"))
+	if (getenv("HATARI_TEST"))
+		psGlobalConfig = NULL;
+	else
+		psGlobalConfig = malloc(FILENAME_MAX);
+	if (psGlobalConfig)
 	{
 		File_MakePathBuf(psGlobalConfig, FILENAME_MAX, CONFDIR,
 		                 "hatari", "cfg");
@@ -885,6 +842,8 @@ static void Main_LoadInitialConfig(void)
 
 	/* Now try the users configuration file */
 	Configuration_Load(NULL);
+	if (ConfigureParams.Keyboard.nLanguage == TOS_LANG_UNKNOWN)
+		ConfigureParams.Keyboard.nLanguage = TOS_DefaultLanguage();
 }
 
 /*-----------------------------------------------------------------------*/
@@ -903,7 +862,7 @@ static void Main_StatusbarSetup(void)
 	};
 	const char *name;
 	bool named;
-	SDLKey key;
+	SDL_Keycode key;
 	int i;
 
 	named = false;
@@ -1001,7 +960,7 @@ int main(int argc, char *argv[])
 		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
 			ConfigureParams.Video.AviRecordFps == 0 ?
 				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
-				(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , ConfigureParams.Video.AviRecordFps ) ,
 			1 << CLOCKS_TIMINGS_SHIFT_VBL ,
 			ConfigureParams.Video.AviRecordVcodec );
 
