@@ -5,7 +5,7 @@
   or at your option any later version. Read the file gpl.txt for details.
 
   Functions for showing and parsing all of Hatari's command line options.
-  
+
   To add a new option:
   - Add option ID to the enum
   - Add the option information to HatariOptions[]
@@ -24,6 +24,7 @@ const char Options_fileid[] = "Hatari options.c";
 #include "version.h"
 #include "options.h"
 #include "configuration.h"
+#include "console.h"
 #include "control.h"
 #include "debugui.h"
 #include "file.h"
@@ -53,20 +54,23 @@ bool AviRecordOnStartup;   /* Start avi recording at startup */
 bool BenchmarkMode;	   /* Start in benchmark mode (try to run at maximum emulation */
 			   /* speed allowed by the CPU). Disable audio/video for best results */
 
-int ConOutDevice = CONOUT_DEVICE_NONE; /* device number for xconout device to track */
-
-static bool bNoSDLParachute, bBiosIntercept;
+static bool bBiosIntercept;
 
 /*  List of supported options. */
 enum {
 	OPT_HEADER,	/* options section header */
+
 	OPT_HELP,		/* general options */
 	OPT_VERSION,
 	OPT_CONFIRMQUIT,
 	OPT_CONFIGFILE,
 	OPT_KEYMAPFILE,
+	OPT_COUNTRY_CODE,
+	OPT_KBD_LAYOUT,
+	OPT_LANGUAGE,
 	OPT_FASTFORWARD,
 	OPT_AUTOSTART,
+
 	OPT_MONO,		/* common display options */
 	OPT_MONITOR,
 	OPT_TOS_RESOLUTION,
@@ -84,17 +88,20 @@ enum {
 	OPT_ZOOM,
 	OPT_FORCEBPP,
 	OPT_DISABLE_VIDEO,
+
 	OPT_BORDERS,		/* ST/STE display options */
-	OPT_RESOLUTION_ST,
 	OPT_SPEC512,
 	OPT_VIDEO_TIMING,
+
 	OPT_RESOLUTION,		/* TT/Falcon display options */
 	OPT_FORCE_MAX,
 	OPT_ASPECT,
+
 	OPT_VDI,		/* VDI options */
 	OPT_VDI_PLANES,
 	OPT_VDI_WIDTH,
 	OPT_VDI_HEIGHT,
+
 	OPT_SCREEN_CROP,        /* screen capture options */
 	OPT_AVIRECORD,
 	OPT_AVIRECORD_VCODEC,
@@ -102,6 +109,7 @@ enum {
 	OPT_AVIRECORD_FPS,
 	OPT_AVIRECORD_FILE,
 	OPT_SCRSHOT_DIR,
+
 	OPT_JOYSTICK,		/* device options */
 	OPT_JOYSTICK0,
 	OPT_JOYSTICK1,
@@ -121,7 +129,8 @@ enum {
 	// OPT_SCCB,
 	// OPT_SCCBIN,
 	OPT_SCCBOUT,
-	OPT_DRIVEA,		/* disk options */
+
+	OPT_DRIVEA,		/* floppy options */
 	OPT_DRIVEB,
 	OPT_DRIVEA_HEADS,
 	OPT_DRIVEB_HEADS,
@@ -129,7 +138,8 @@ enum {
 	OPT_DISKB,
 	OPT_FASTFLOPPY,
 	OPT_WRITEPROT_FLOPPY,
-	OPT_HARDDRIVE,
+
+	OPT_HARDDRIVE,		/* HD options */
 	OPT_WRITEPROT_HD,
 	OPT_GEMDOS_CASE,
 	OPT_GEMDOS_HOSTTIME,
@@ -140,43 +150,44 @@ enum {
 	OPT_IDEMASTERHDIMAGE,
 	OPT_IDESLAVEHDIMAGE,
 	OPT_IDEBYTESWAP,
+
 	OPT_MEMSIZE,		/* memory options */
-#if ENABLE_WINUAE_CPU
 	OPT_TT_RAM,
-#endif
 	OPT_MEMSTATE,
+
 	OPT_TOS,		/* ROM options */
 	OPT_PATCHTOS,
 	OPT_CARTRIDGE,
+
 	OPT_CPULEVEL,		/* CPU options */
 	OPT_CPUCLOCK,
 	OPT_COMPATIBLE,
-#if ENABLE_WINUAE_CPU
-	OPT_CPU_CYCLE_EXACT,	/* WinUAE CPU/FPU/bus options */
+	OPT_CPU_CYCLE_EXACT,
 	OPT_CPU_ADDR24,
 	OPT_FPU_TYPE,
 /*	OPT_FPU_JIT_COMPAT, */
 	OPT_FPU_SOFTFLOAT,
 	OPT_MMU,
-#endif
+
 	OPT_MACHINE,		/* system options */
 	OPT_BLITTER,
+	OPT_VME,
 	OPT_DSP,
 	OPT_TIMERD,
 	OPT_FASTBOOT,
+
 	OPT_MICROPHONE,		/* sound options */
 	OPT_SOUND,
 	OPT_SOUNDBUFFERSIZE,
 	OPT_SOUNDSYNC,
 	OPT_YM_MIXING,
+
 #ifdef WIN32
 	OPT_WINCON,		/* debug options */
 #endif
 	OPT_DEBUG,
 	OPT_EXCEPTIONS,
-#if ENABLE_WINUAE_CPU
 	OPT_LILO,
-#endif
 	OPT_BIOSINTERCEPT,
 	OPT_CONOUT,
 	OPT_DISASM,
@@ -185,7 +196,6 @@ enum {
 	OPT_TRACEFILE,
 	OPT_PARSE,
 	OPT_SAVECONFIG,
-	OPT_PARACHUTE,
 	OPT_CONTROLSOCKET,
 	OPT_CMDFIFO,
 	OPT_LOGFILE,
@@ -207,7 +217,7 @@ typedef struct {
 
 /* it's easier to edit these if they are kept in the same order as the enums */
 static const opt_t HatariOptions[] = {
-	
+
 	{ OPT_HEADER, NULL, NULL, NULL, "General" },
 	{ OPT_HELP,      "-h", "--help",
 	  NULL, "Print this help text and exit" },
@@ -219,6 +229,12 @@ static const opt_t HatariOptions[] = {
 	  "<file>", "Read (additional) configuration values from <file>" },
 	{ OPT_KEYMAPFILE, "-k", "--keymap",
 	  "<file>", "Read (additional) keyboard mappings from <file>" },
+	{ OPT_COUNTRY_CODE, NULL, "--country",
+	  "<x>", "Set country code for multi-code EmuTOS ROM" },
+	{ OPT_KBD_LAYOUT, NULL, "--layout",
+	  "<x>", "Set (TT/Falcon) NVRAM keyboard layout" },
+	{ OPT_LANGUAGE, NULL, "--language",
+	  "<x>", "Set (TT/Falcon) NVRAM language" },
 	{ OPT_FASTFORWARD, NULL, "--fast-forward",
 	  "<bool>", "Help skipping stuff on fast machine" },
 	{ OPT_AUTOSTART, NULL, "--auto",
@@ -249,21 +265,12 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Show statusbar (floppy leds etc)" },
 	{ OPT_DRIVE_LED,   NULL, "--drive-led",
 	  "<bool>", "Show overlay drive led when statusbar isn't shown" },
-#if WITH_SDL2
 	{ OPT_MAXWIDTH, NULL, "--max-width",
 	  "<x>", "Maximum Hatari screen width before scaling" },
 	{ OPT_MAXHEIGHT, NULL, "--max-height",
 	  "<x>", "Maximum Hatari screen height before scaling" },
 	{ OPT_ZOOM, "-z", "--zoom",
 	  "<x>", "Hatari screen/window scaling factor (1.0 - 8.0)" },
-#else
-	{ OPT_MAXWIDTH, NULL, "--max-width",
-	  "<x>", "Maximum window width for borders & zooming" },
-	{ OPT_MAXHEIGHT, NULL, "--max-height",
-	  "<x>", "Maximum window height for borders & zooming" },
-	{ OPT_ZOOM, "-z", "--zoom",
-	  "<x>", "Double small ST/STe resolutions (1=no, 2=yes)" },
-#endif
 	{ OPT_FORCEBPP, NULL, "--bpp",
 	  "<x>", "Force internal bitdepth (x = 15/16/32, 0=disable)" },
 	{ OPT_DISABLE_VIDEO,   NULL, "--disable-video",
@@ -272,8 +279,6 @@ static const opt_t HatariOptions[] = {
 	{ OPT_HEADER, NULL, NULL, NULL, "ST/STE specific display" },
 	{ OPT_BORDERS, NULL, "--borders",
 	  "<bool>", "Show screen borders (for overscan demos etc)" },
-	{ OPT_RESOLUTION_ST, NULL, "--desktop-st",
-	  "<bool>", "Keep desktop resolution on fullscreen" },
 	{ OPT_SPEC512, NULL, "--spec512",
 	  "<x>", "Spec512 palette threshold (0 <= x <= 512, 0=disable)" },
 	{ OPT_VIDEO_TIMING,   NULL, "--video-timing",
@@ -400,10 +405,8 @@ static const opt_t HatariOptions[] = {
 	{ OPT_HEADER, NULL, NULL, NULL, "Memory" },
 	{ OPT_MEMSIZE,   "-s", "--memsize",
 	  "<x>", "ST RAM size (x = size in MiB from 0 to 14, 0 = 512KiB ; else size in KiB)" },
-#if ENABLE_WINUAE_CPU
 	{ OPT_TT_RAM,   NULL, "--ttram",
-	  "<x>", "TT RAM size (x = size in MiB from 0 to 512)" },
-#endif
+	  "<x>", "TT RAM size (x = size in MiB from 0 to 1024, in steps of 4)" },
 	{ OPT_MEMSTATE,   NULL, "--memstate",
 	  "<file>", "Load memory snap-shot <file>" },
 
@@ -415,18 +418,13 @@ static const opt_t HatariOptions[] = {
 	{ OPT_CARTRIDGE, NULL, "--cartridge",
 	  "<file>", "Use ROM cartridge image <file>" },
 
-#if ENABLE_WINUAE_CPU
 	{ OPT_HEADER, NULL, NULL, NULL, "CPU/FPU/bus" },
-#else
-	{ OPT_HEADER, NULL, NULL, NULL, "CPU" },
-#endif
 	{ OPT_CPULEVEL,  NULL, "--cpulevel",
 	  "<x>", "Set the CPU type (x => 680x0) (EmuTOS/TOS 2.06 only!)" },
 	{ OPT_CPUCLOCK,  NULL, "--cpuclock",
 	  "<x>", "Set the CPU clock (x = 8/16/32)" },
 	{ OPT_COMPATIBLE, NULL, "--compatible",
 	  "<bool>", "Use a more compatible (but slower) prefetch mode for CPU" },
-#if ENABLE_WINUAE_CPU
 	{ OPT_CPU_CYCLE_EXACT, NULL, "--cpu-exact",
 	  "<bool>", "Use cycle exact CPU emulation" },
 	{ OPT_CPU_ADDR24, NULL, "--addr24",
@@ -439,7 +437,6 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Use full software FPU emulation" },
 	{ OPT_MMU, NULL, "--mmu",
 	  "<bool>", "Use MMU emulation" },
-#endif
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Misc system" },
 	{ OPT_MACHINE,   NULL, "--machine",
@@ -448,6 +445,8 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Use blitter emulation (ST only)" },
 	{ OPT_DSP,       NULL, "--dsp",
 	  "<x>", "DSP emulation (x = none/dummy/emu, Falcon only)" },
+	{ OPT_VME,	NULL, "--vme",
+	  "<x>", "VME mode (x = none/dummy, MegaSTE/TT only)" },
 	{ OPT_TIMERD,    NULL, "--timer-d",
 	  "<bool>", "Patch Timer-D (about doubles ST emulation speed)" },
 	{ OPT_FASTBOOT, NULL, "--fast-boot",
@@ -474,9 +473,7 @@ static const opt_t HatariOptions[] = {
 	  NULL, "Toggle whether CPU exceptions invoke debugger" },
 	{ OPT_EXCEPTIONS, NULL, "--debug-except",
 	  "<flags>", "Exceptions invoking debugger, see '--debug-except help'" },
-#if ENABLE_WINUAE_CPU
 	{ OPT_LILO, NULL, "--lilo", "<x>", "Boot Linux (see manual page)" },
-#endif
 	{ OPT_BIOSINTERCEPT, NULL, "--bios-intercept",
 	  "<bool>", "Enable/disable XBIOS command parsing support" },
 	{ OPT_CONOUT,   NULL, "--conout",
@@ -493,8 +490,6 @@ static const opt_t HatariOptions[] = {
 	  "<file>", "Parse/execute debugger commands from <file>" },
 	{ OPT_SAVECONFIG, NULL, "--saveconfig",
 	  NULL, "Save current Hatari configuration and exit" },
-	{ OPT_PARACHUTE, NULL, "--no-parachute",
-	  NULL, "Disable SDL parachute to get Hatari core dumps" },
 #if HAVE_UNIX_DOMAIN_SOCKETS
 	{ OPT_CONTROLSOCKET, NULL, "--control-socket",
 	  "<file>", "Hatari connects to given socket for commands" },
@@ -607,7 +602,7 @@ static const opt_t *Opt_ShowHelpSection(const opt_t *start_opt)
 		}
 	}
 	last = opt;
-	
+
 	/* output all options */
 	for (opt = start_opt; opt != last; opt++)
 	{
@@ -703,7 +698,6 @@ bool Opt_ShowError(unsigned int optid, const char *value, const char *error)
  */
 int Opt_ValueAlignMinMax(int value, int align, int min, int max)
 {
-	value = (value/align)*align;
 	if (value > max)
 	{
 		/* align down */
@@ -715,7 +709,7 @@ int Opt_ValueAlignMinMax(int value, int align, int min, int max)
 		min += align-1;
 		return (min/align)*align;
 	}
-	return value;
+	return (value/align)*align;
 }
 
 
@@ -766,6 +760,23 @@ static bool Opt_Bool(const char *arg, int optid, bool *conf)
 	return Opt_ShowError(optid, orig, "Not a <bool> value");
 }
 
+/**
+ * If 'conf' given, set it to parsed country code value
+ * Return false for any other value, otherwise true
+ */
+static bool Opt_CountryCode(const char *arg, int optid, int *conf)
+{
+	int val = TOS_ParseCountryCode(arg);
+	if (val != TOS_LANG_UNKNOWN)
+	{
+		*conf = val;
+		return true;
+	}
+	Opt_ShowError(optid, arg, "Invalid value");
+	TOS_ShowCountryCodes();
+	return false;
+
+}
 
 /**
  * checks str argument against options of type "--option<digit>".
@@ -820,7 +831,7 @@ static int Opt_CheckBracketValue(const opt_t *opt, const char *str)
  * matches string under given index in the argv against all Hatari
  * short and long options. If match is found, returns ID for that,
  * otherwise shows help and returns OPT_ERROR.
- * 
+ *
  * Checks also that if option is supposed to have argument,
  * whether there's one.
  */
@@ -831,7 +842,7 @@ static int Opt_WhichOption(int argc, const char * const argv[], int idx)
 	int id;
 
 	for (opt = HatariOptions; opt->id != OPT_ERROR; opt++)
-	{	
+	{
 		/* exact option name matches? */
 		if (!((opt->str && !strcmp(str, opt->str)) ||
 		      (opt->chr && !strcmp(str, opt->chr))))
@@ -907,21 +918,6 @@ static bool Opt_StrCpy(int optid, bool checkexist, char *dst, const char *src, s
 }
 
 
-/**
- * Return SDL_INIT_NOPARACHUTE flag if user requested SDL parachute
- * to be disabled to get proper Hatari core dumps.  By default returns
- * zero so that SDL parachute will be used to restore video mode on
- * unclean Hatari termination.
- */
-Uint32 Opt_GetNoParachuteFlag(void)
-{
-	if (bNoSDLParachute)
-		return SDL_INIT_NOPARACHUTE;
-
-	return 0;
-}
-
-
 /**
  * Do final validation for the earlier + parsed options
  *
@@ -1053,14 +1049,11 @@ static bool Opt_HandleArgument(const char *path)
  */
 bool Opt_ParseParameters(int argc, const char * const argv[])
 {
-	int ncpu, skips, planes, cpuclock, threshold, memsize, port, freq, temp, drive;
+	int ncpu, skips, planes, cpuclock, threshold, memsize;
+	int dev, port, freq, temp, drive;
 	const char *errstr, *str;
 	int i, ok = true;
-#if WITH_SDL2
 	float zoom;
-#else
-	int zoom;
-#endif
 	int val;
 
 	/* Defaults for loading initial memory snap-shots */
@@ -1167,12 +1160,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 
 		case OPT_RESIZABLE:
-#if WITH_SDL2
 			ok = Opt_Bool(argv[++i], OPT_RESIZABLE, &ConfigureParams.Screen.bResizable);
-#else
-			fprintf(stderr, "The --resizable option is supported only in SDL2 build!\n");
-			i++;
-#endif
 			break;
 
 		case OPT_FRAMESKIPS:
@@ -1238,15 +1226,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ok = Opt_Bool(argv[++i], OPT_BORDERS, &ConfigureParams.Screen.bAllowOverscan);
 			break;
 
-		case OPT_RESOLUTION_ST:
-#if WITH_SDL2
-			fprintf(stderr, "The --desktop-st option is not supported in SDL2 build!\n");
-			i++;
-#else
-			ok = Opt_Bool(argv[++i], OPT_RESOLUTION_ST, &ConfigureParams.Screen.bKeepResolutionST);
-#endif
-			break;
-			
 		case OPT_SPEC512:
 			threshold = atoi(argv[++i]);
 			if (threshold < 0 || threshold > 512)
@@ -1259,19 +1238,13 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			break;
 
 		case OPT_ZOOM:
-#if WITH_SDL2
 			zoom = atof(argv[++i]);
 			if (zoom < 1.0 || zoom > 8.0)
-#else
-			zoom = atoi(argv[++i]);
-			if (zoom < 1 || zoom > 2)
-#endif
 			{
 				return Opt_ShowError(OPT_ZOOM, argv[i], "Invalid zoom value");
 			}
 			ConfigureParams.Screen.nMaxWidth = NUM_VISIBLE_LINE_PIXELS;
 			ConfigureParams.Screen.nMaxHeight = NUM_VISIBLE_LINES;
-#if WITH_SDL2
 			/* double ST-low always so that resulting screen size
 			 * is approximately same size with same zoom factor
 			 * regardless of the machine or monitor type
@@ -1279,14 +1252,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.Screen.nMaxWidth *= 2;
 			ConfigureParams.Screen.nMaxHeight *= 2;
 			ConfigureParams.Screen.nZoomFactor = zoom;
-#else
-			/* zoom factor only for ST-low mode */
-			if (zoom > 1)
-			{
-				ConfigureParams.Screen.nMaxWidth *= 2;
-				ConfigureParams.Screen.nMaxHeight *= 2;
-			}
-#endif
 			ConfigureParams.Screen.nMaxHeight += STATUSBAR_MAX_HEIGHT;
 			break;
 
@@ -1310,7 +1275,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_RESOLUTION:
 			ok = Opt_Bool(argv[++i], OPT_RESOLUTION, &ConfigureParams.Screen.bKeepResolution);
 			break;
-			
+
 		case OPT_MAXWIDTH:
 			ConfigureParams.Screen.nMaxWidth = atoi(argv[++i]);
 			break;
@@ -1322,7 +1287,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_FORCE_MAX:
 			ok = Opt_Bool(argv[++i], OPT_FORCE_MAX, &ConfigureParams.Screen.bForceMax);
 			break;
-			
+
 		case OPT_ASPECT:
 			ok = Opt_Bool(argv[++i], OPT_ASPECT, &ConfigureParams.Screen.bAspectCorrect);
 			break;
@@ -1444,7 +1409,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_JOYSTICK0, argv[i], "Invalid joystick port");
 			}
 			i += 1;
-			if (strcasecmp(argv[i], "none") == 0)
+			if (strcasecmp(argv[i], "none") == 0 || strcasecmp(argv[i], "off") == 0)
 			{
 				ConfigureParams.Joysticks.Joy[port].nJoystickMode = JOYSTICK_DISABLED;
 			}
@@ -1481,7 +1446,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 					argv[i], sizeof(ConfigureParams.Midi.sMidiInFileName),
 					&ConfigureParams.Midi.bEnableMidi);
 			break;
-			
+
 		case OPT_MIDI_OUT:
 			i += 1;
 			ok = Opt_StrCpy(OPT_MIDI_OUT, false, ConfigureParams.Midi.sMidiOutFileName,
@@ -1767,13 +1732,11 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bLoadAutoSave = false;
 			break;
 
-#if ENABLE_WINUAE_CPU
 		case OPT_TT_RAM:
 			memsize = atoi(argv[++i]);
-			ConfigureParams.Memory.TTRamSize_KB = Opt_ValueAlignMinMax(memsize+3, 4, 0, 512) * 1024;
+			ConfigureParams.Memory.TTRamSize_KB = Opt_ValueAlignMinMax(memsize+3, 4, 0, 1024) * 1024;
 			bLoadAutoSave = false;
 			break;
-#endif
 
 		case OPT_TOS:
 			i += 1;
@@ -1817,11 +1780,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_CPULEVEL:
 			/* UAE core uses cpu_level variable */
 			ncpu = atoi(argv[++i]);
-#if ENABLE_WINUAE_CPU
 			if(ncpu < 0 || ncpu == 5 || ncpu > 6)
-#else
-			if(ncpu < 0 || ncpu > 4)
-#endif
 			{
 				return Opt_ShowError(OPT_CPULEVEL, argv[i], "Invalid CPU level");
 			}
@@ -1848,7 +1807,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				bLoadAutoSave = false;
 			}
 			break;
-#if ENABLE_WINUAE_CPU
 		case OPT_CPU_ADDR24:
 			ok = Opt_Bool(argv[++i], OPT_CPU_ADDR24, &ConfigureParams.System.bAddressSpace24);
 			bLoadAutoSave = false;
@@ -1861,7 +1819,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_FPU_TYPE:
 			i += 1;
-			if (strcasecmp(argv[i], "none") == 0)
+			if (strcasecmp(argv[i], "none") == 0 || strcasecmp(argv[i], "off") == 0)
 			{
 				ConfigureParams.System.n_FPUType = FPU_NONE;
 			}
@@ -1896,7 +1854,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ok = Opt_Bool(argv[++i], OPT_MMU, &ConfigureParams.System.bMMU);
 			bLoadAutoSave = false;
 			break;
-#endif
 
 			/* system options */
 		case OPT_MACHINE:
@@ -1944,7 +1901,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				return Opt_ShowError(OPT_MACHINE, argv[i], "Unknown machine type");
 			}
-#if ENABLE_WINUAE_CPU
 			if (Config_IsMachineST() || Config_IsMachineSTE())
 			{
 				ConfigureParams.System.bMMU = false;
@@ -1959,7 +1915,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				ConfigureParams.System.n_FPUType = FPU_NONE;	/* TODO: or leave it as-is? */
 			}
-#endif
 			bLoadAutoSave = false;
 			break;
 
@@ -1980,7 +1935,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_DSP:
 			i += 1;
-			if (strcasecmp(argv[i], "none") == 0)
+			if (strcasecmp(argv[i], "none") == 0 || strcasecmp(argv[i], "off") == 0)
 			{
 				ConfigureParams.System.nDSPType = DSP_TYPE_NONE;
 			}
@@ -2003,6 +1958,23 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bLoadAutoSave = false;
 			break;
 
+		case OPT_VME:
+			i += 1;
+			if (strcasecmp(argv[i], "dummy") == 0)
+			{
+				ConfigureParams.System.nVMEType = VME_TYPE_DUMMY;
+			}
+			else if (strcasecmp(argv[i], "none") == 0 || strcasecmp(argv[i], "off") == 0)
+			{
+				ConfigureParams.System.nVMEType = VME_TYPE_NONE;
+			}
+			else
+			{
+				return Opt_ShowError(OPT_VME, argv[i], "Unknown VME type");
+			}
+			bLoadAutoSave = false; /* TODO: needed? */
+			break;
+
 			/* sound options */
 		case OPT_YM_MIXING:
 			i += 1;
@@ -2061,11 +2033,23 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_SOUNDSYNC:
 			ok = Opt_Bool(argv[++i], OPT_SOUNDSYNC, &ConfigureParams.Sound.bEnableSoundSync);
 			break;
-			
+
 		case OPT_MICROPHONE:
 			ok = Opt_Bool(argv[++i], OPT_MICROPHONE, &ConfigureParams.Sound.bEnableMicrophone);
 			break;
 
+		case OPT_COUNTRY_CODE:
+			ok = Opt_CountryCode(argv[++i], OPT_COUNTRY_CODE, &ConfigureParams.Keyboard.nCountryCode);
+			break;
+
+		case OPT_LANGUAGE:
+			ok = Opt_CountryCode(argv[++i], OPT_LANGUAGE, &ConfigureParams.Keyboard.nLanguage);
+			break;
+
+		case OPT_KBD_LAYOUT:
+			ok = Opt_CountryCode(argv[++i], OPT_KBD_LAYOUT, &ConfigureParams.Keyboard.nKbdLayout);
+			break;
+
 		case OPT_KEYMAPFILE:
 			i += 1;
 			ok = Opt_StrCpy(OPT_KEYMAPFILE, true, ConfigureParams.Keyboard.szMappingFileName,
@@ -2076,7 +2060,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				ConfigureParams.Keyboard.nKeymapType = KEYMAP_LOADED;
 			}
 			break;
-			
+
 			/* debug options */
 #ifdef WIN32
 		case OPT_WINCON:
@@ -2118,7 +2102,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				        oldmask, ExceptionDebugMask);
 			}
 			break;
-#if ENABLE_WINUAE_CPU
+
 		case OPT_LILO: {
 			size_t len;
 			len = strlen(argv[++i]);
@@ -2137,7 +2121,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bUseTos = false;
 			break;
 		}
-#endif
+
 		case OPT_BIOSINTERCEPT:
 			ok = Opt_Bool(argv[++i], OPT_BIOSINTERCEPT, &bBiosIntercept);
 			Log_Printf(LOG_DEBUG, "XBIOS 11/20/255 Hatari versions %sabled: "
@@ -2148,12 +2132,12 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_CONOUT:
 			i += 1;
-			ConOutDevice = atoi(argv[i]);
-			if (ConOutDevice < 0 || ConOutDevice > 7)
+			dev = atoi(argv[i]);
+			if (!Console_SetDevice(dev))
 			{
 				return Opt_ShowError(OPT_CONOUT, argv[i], "Invalid console device vector number");
 			}
-			Log_Printf(LOG_DEBUG, "Xcounout device %d vector redirection enabled.\n", ConOutDevice);
+			Log_Printf(LOG_DEBUG, "Xcounout device %d vector redirection enabled.\n", dev);
 			break;
 
 		case OPT_NATFEATS:
@@ -2161,10 +2145,6 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			Log_Printf(LOG_DEBUG, "Native Features %s.\n", ConfigureParams.Log.bNatFeats ? "enabled" : "disabled");
 			break;
 
-		case OPT_PARACHUTE:
-			bNoSDLParachute = true;
-			break;
-
 		case OPT_DISASM:
 			i += 1;
 			errstr = Disasm_ParseOption(argv[i]);
@@ -2178,7 +2158,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_DISASM, argv[i], errstr);
 			}
 			break;
-			
+
 		case OPT_TRACE:
 			i += 1;
 			errstr = Log_SetTraceOptions(argv[i]);
@@ -2261,7 +2241,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			Log_Printf(LOG_DEBUG, "Exit after %d VBLs.\n", val);
 			Main_SetRunVBLs(val);
 			break;
-		       
+
 		case OPT_BENCHMARK:
 			BenchmarkMode = true;
 			break;
@@ -2292,7 +2272,7 @@ char *Opt_MatchOption(const char *text, int state)
 {
 	static int i, len;
 	const char *name;
-	
+
 	if (!state)
 	{
 		/* first match */
