@@ -20,6 +20,7 @@ const char Reset_fileid[] = "Hatari reset.c";
 #include "acia.h"
 #include "ikbd.h"
 #include "ioMem.h"
+#include "cycles.h"
 #include "cycInt.h"
 #include "m68000.h"
 #include "mfp.h"
@@ -36,6 +37,7 @@ const char Reset_fileid[] = "Hatari reset.c";
 #include "vdi.h"
 #include "nvram.h"
 #include "video.h"
+#include "vme.h"
 #include "falcon/videl.h"
 #include "falcon/dsp.h"
 #include "debugcpu.h"
@@ -84,7 +86,10 @@ static int Reset_ST(bool bCold)
 	{
 		Ncr5380_Reset();
 	}
-
+	if (Config_IsMachineTT() || Config_IsMachineMegaSTE())
+	{
+		VME_Reset();
+	}
 	if (Config_IsMachineFalcon())
 	{
 		DSP_Reset();                  /* Reset the DSP */
