@@ -51,6 +51,7 @@ const char Statusbar_fileid[] = "Hatari statusbar.c";
 #include "stMemory.h"
 #include "blitter.h"
 #include "str.h"
+#include "lilo.h"
 
 #define DEBUG 0
 #if DEBUG
@@ -541,18 +542,12 @@ void Statusbar_UpdateInfo(void)
 	}
 
 	/* Prefetch mode or cycle exact mode ? */
-#if ENABLE_WINUAE_CPU
 	if ( ConfigureParams.System.bCycleExactCpu )
 		end = Statusbar_AddString(end, "(CE)");
 	else if ( ConfigureParams.System.bCompatibleCpu )
 		end = Statusbar_AddString(end, "(PF)");
-#else
-	if ( ConfigureParams.System.bCompatibleCpu )
-		end = Statusbar_AddString(end, "(PF)");
-#endif
 
 	/* additional WinUAE CPU/FPU info */
-#if ENABLE_WINUAE_CPU
 	*end++ = '/';
 	switch (ConfigureParams.System.n_FPUType)
 	{
@@ -576,7 +571,6 @@ void Statusbar_UpdateInfo(void)
 	{
 		end = Statusbar_AddString(end, "/MMU");
 	}
-#endif
 
 	/* amount of memory in MB */
 	*end++ = ' ';
@@ -639,6 +633,10 @@ void Statusbar_UpdateInfo(void)
 			end = Statusbar_AddString(end, "EmuTOS");
 		}
 	}
+	else if (bUseLilo)
+	{
+		end = Statusbar_AddString(end, "Linux");
+	}
 	else
 	{
 		end = Statusbar_AddString(end, "TOS ");
