@@ -30,6 +30,7 @@ const char TOS_fileid[] = "Hatari tos.c";
 #include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
+#include "nvram.h"
 #include "stMemory.h"
 #include "str.h"
 #include "tos.h"
@@ -750,12 +751,9 @@ static void TOS_FixRom(Uint32 *logopatch_addr)
 		if (pPatch->Version == TosVersion
 		    && (pPatch->Country == TosCountry || pPatch->Country == -1))
 		{
-#if ENABLE_WINUAE_CPU
 			bool use_mmu = ConfigureParams.System.bMMU &&
 			               ConfigureParams.System.nCpuLevel == 3;
-#else
-			bool use_mmu = false;
-#endif
+
 			/* Only apply the patch if it is really needed: */
 			if (pPatch->Flags == TP_ALWAYS
 			    || (pPatch->Flags == TP_HDIMAGE_OFF && !bAcsiEmuOn
@@ -812,9 +810,7 @@ static void TOS_CheckSysConfig(void)
 {
 	int oldCpuLevel = ConfigureParams.System.nCpuLevel;
 	MACHINETYPE oldMachineType = ConfigureParams.System.nMachineType;
-#if ENABLE_WINUAE_CPU
 	FPUTYPE oldFpuType = ConfigureParams.System.n_FPUType;
-#endif
 
 	if (((TosVersion == 0x0106 || TosVersion == 0x0162) && ConfigureParams.System.nMachineType != MACHINE_STE)
 	    || (TosVersion == 0x0162 && ConfigureParams.System.nCpuLevel != 0))
@@ -895,7 +891,6 @@ static void TOS_CheckSysConfig(void)
 		             " ==> Switching to 68020 mode now.\n");
 		ConfigureParams.System.nCpuLevel = 2;
 	}
-#if ENABLE_WINUAE_CPU
 	else if ((TosVersion & 0x0f00) == 0x0300 &&
 	         (ConfigureParams.System.nCpuLevel < 2 || ConfigureParams.System.n_FPUType == FPU_NONE))
 	{
@@ -904,19 +899,10 @@ static void TOS_CheckSysConfig(void)
 		ConfigureParams.System.nCpuLevel = 3;
 		ConfigureParams.System.n_FPUType = FPU_68882;
 	}
-#else
-	else if ((TosVersion & 0x0f00) == 0x0300 && ConfigureParams.System.nCpuLevel < 3)
-	{
-		Log_AlertDlg(LOG_ERROR, "TOS versions 3.0x require a CPU >= 68020 with FPU.\n"
-		             " ==> Switching to 68030 mode with FPU now.\n");
-		ConfigureParams.System.nCpuLevel = 3;
-	}
-#endif
 
 	/* TOS version triggered changes? */
 	if (ConfigureParams.System.nMachineType != oldMachineType)
 	{
-#if ENABLE_WINUAE_CPU
 		if (Config_IsMachineTT())
 		{
 			ConfigureParams.System.bCompatibleFPU = true;
@@ -929,14 +915,10 @@ static void TOS_CheckSysConfig(void)
 			ConfigureParams.System.bAddressSpace24 = true;
 			ConfigureParams.System.bMMU = false;
 		}
-#endif
 		M68000_CheckCpuSettings();
 	}
 	else if (ConfigureParams.System.nCpuLevel != oldCpuLevel
-#if ENABLE_WINUAE_CPU
-		 || ConfigureParams.System.n_FPUType != oldFpuType
-#endif
-		)
+		 || ConfigureParams.System.n_FPUType != oldFpuType)
 	{
 		M68000_CheckCpuSettings();
 	}
@@ -1023,13 +1005,14 @@ static uint8_t *TOS_LoadImage(void)
 		 * just for fun. */
 		TosAddress = 0xfc0000;
 	}
-	else if (TosVersion < 0x100 || TosVersion >= 0x500 || TosSize > 1024*1024L
+	else if (TosVersion < 0x100 || TosVersion >= 0x700 || TosSize > 1024*1024L
 	         || (TosAddress == 0xfc0000 && TosSize > 224*1024L)
 	         || (bRamTosImage && TosAddress + TosSize > STRamEnd)
 	         || (!bRamTosImage && TosAddress != 0xe00000 && TosAddress != 0xfc0000))
 	{
 		Log_AlertDlg(LOG_FATAL, "Your TOS image seems not to be a valid TOS ROM file!\n"
-		             "(TOS version %x, address $%x)", TosVersion, TosAddress);
+		             "(TOS version %x.%02x, address $%x)",
+			     TosVersion >> 8, TosVersion & 0xff, TosAddress);
 		free(pTosFile);
 		return NULL;
 	}
@@ -1038,10 +1021,9 @@ static uint8_t *TOS_LoadImage(void)
 	 * 512k EmuTOS declares itself as TOS 2.x, but it can handle
 	 * all machine types, so it can & needs to be skipped here
 	 */
-	if (!(bIsEmuTOS && TosSize == 512*1024))
+	if (!bIsEmuTOS || TosSize < 512 * 1024)
 		TOS_CheckSysConfig();
 
-#if ENABLE_WINUAE_CPU
 	/* 32-bit addressing is supported only by CPU >= 68020, TOS v3, TOS v4 and EmuTOS */
 	if (ConfigureParams.System.nCpuLevel < 2 || (TosVersion < 0x0300 && !bIsEmuTOS))
 	{
@@ -1073,7 +1055,6 @@ static uint8_t *TOS_LoadImage(void)
 			break;
 		}
 	}
-#endif
 
 	return pTosFile;
 }
@@ -1123,6 +1104,7 @@ int TOS_InitImage(void)
 {
 	uint8_t *pTosFile = NULL;
 	Uint32 logopatch_addr = 0;
+	Uint16 osconf, countrycode;
 
 	bTosImageLoaded = false;
 
@@ -1199,6 +1181,21 @@ int TOS_InitImage(void)
 		Log_Printf(LOG_DEBUG, "Skipped TOS patches.\n");
 	}
 
+	/* whether to override EmuTOS country code */
+	osconf = STMemory_ReadWord(TosAddress+0x1C);
+	countrycode = osconf >> 1;
+	if (bIsEmuTOS && countrycode == TOS_LANG_ALL && !NvRam_Present() &&
+	    ConfigureParams.Keyboard.nCountryCode != TOS_LANG_UNKNOWN)
+	{
+		countrycode = ConfigureParams.Keyboard.nLanguage;
+		/* low bit: us -> NTSC (0), any other -> PAL (1) */
+		osconf = (countrycode << 1) | (countrycode?1:0);
+		STMemory_WriteWord(TosAddress+0x1C, osconf);
+		Log_Printf(LOG_WARN, "=> EmuTOS country code: %d (%s), %s\n",
+			   countrycode, TOS_LanguageName(countrycode),
+			   (osconf & 1) ? "PAL" : "NTSC");
+	}
+
 	/*
 	 * patch some values into the "Draw logo" patch.
 	 * Needs to be called after final VDI resolution has been determined.
@@ -1242,3 +1239,92 @@ int TOS_InitImage(void)
 	bTosImageLoaded = true;
 	return 0;
 }
+
+static const struct {
+	uint8_t value;
+	const char *code;
+	const char *name;
+} countries[] = {
+	{ TOS_LANG_US,    "us",    "USA" },
+	{ TOS_LANG_DE,    "de",    "Germany" },
+	{ TOS_LANG_FR,    "fr",    "France" },
+	{ TOS_LANG_UK,    "uk",    "United Kingdom" },
+	{ TOS_LANG_ES,    "es",    "Spain" },
+	{ TOS_LANG_IT,    "it",    "Italy" },
+	{ TOS_LANG_SE,    "se",    "Sweden" },
+	{ TOS_LANG_CH_FR, "ch_fr", "Switzerland (French)" },
+	{ TOS_LANG_CH_DE, "ch_de", "Switzerland (German)" },
+	{ TOS_LANG_TR,    "tr",    "Turkey" },
+	{ TOS_LANG_FI,    "fi",    "Finland" },
+	{ TOS_LANG_NO,    "no",    "Norway" },
+	{ TOS_LANG_DK,    "dk",    "Denmark" },
+	{ TOS_LANG_SA,    "sa",    "Saudi Arabia" },
+	{ TOS_LANG_NL,    "nl",    "Holland" },
+	{ TOS_LANG_CS,    "cs",    "Czech Republic" },
+	{ TOS_LANG_HU,    "hu",    "Hungary" },
+	{ TOS_LANG_RU,    "ru",    "Russia" },
+	{ TOS_LANG_GR,    "gr",    "Greece" },
+};
+
+/**
+ * TOS_ValidCountryCode: returns parsed country code if
+ * it's recognized, otherwise TOS_LANG_UNKNOWN is returned.
+ */
+int TOS_ParseCountryCode(const char *code)
+{
+	for (int i = 0; i < ARRAY_SIZE(countries); i++) {
+		if (strcmp(code, countries[i].code) == 0) {
+			return countries[i].value;
+		}
+	}
+	return TOS_LANG_UNKNOWN;
+}
+
+void TOS_ShowCountryCodes(void)
+{
+	fprintf(stderr, "\nTOS v4 supports:\n");
+	for (int i = 0; i < ARRAY_SIZE(countries); i++) {
+		if (i == 7)
+			fprintf(stderr, "\nEmuTOS 1024k (v1.1.x) supports also:\n");
+		fprintf(stderr, "- %s : %s\n",
+			countries[i].code, countries[i].name);
+	}
+}
+
+/**
+ * TOS_DefaultLanguage: return TOS country code matching LANG
+ * environment variable. Supports LANG formats: "uk", "en_UK.*"
+ */
+int TOS_DefaultLanguage(void)
+{
+	int len;
+	const char *lang = getenv("LANG");
+	if (!lang)
+		return TOS_LANG_UNKNOWN;
+
+	len = strlen(lang);
+	if (len == 2)
+		return TOS_ParseCountryCode(lang);
+
+	if (len >= 5 && lang[2] == '_') {
+		char cc[3];
+		cc[0] = tolower(lang[3]);
+		cc[1] = tolower(lang[4]);
+		cc[2] = '\0';
+		return TOS_ParseCountryCode(cc);
+	}
+	return TOS_LANG_UNKNOWN;
+}
+
+/**
+ * TOS_LanguageName: return name for given country code
+ */
+const char *TOS_LanguageName(int code)
+{
+	for (int i = 0; i < ARRAY_SIZE(countries); i++) {
+		if (code == countries[i].value) {
+			return countries[i].name;
+		}
+	}
+	return "Unknown";
+}
