@@ -89,14 +89,7 @@ void M68000_SetDebugger(bool debug) { }
 /* fake UAE core registers */
 #include "newcpu.h"
 struct regstruct regs;
-#if ENABLE_WINUAE_CPU
-void m68k_dumpstate(uaecptr *nextpc, uaecptr prevpc) { }
 void m68k_dumpstate_file (FILE *f, uaecptr *nextpc, uaecptr prevpc) { }
-void m68k_disasm(uaecptr addr, uaecptr *nextpc, uaecptr lastpc, int cnt) { }
-#else
-void m68k_dumpstate (FILE *f, uaecptr *nextpc) { }
-void m68k_disasm (FILE *f, uaecptr addr, uaecptr *nextpc, int cnt) { }
-#endif
 
 /* fake debugui.c stuff */
 #include "debug_priv.h"
@@ -133,6 +126,7 @@ info_func_t DebugInfo_GetInfoFunc(const char *name) {
 #ifdef ENABLE_DSP_EMU
 #include "debugdsp.h"
 void DebugDsp_InitSession(void) { }
+Uint32 DebugDsp_CallDepth(void) { return 0; }
 Uint32 DebugDsp_InstrCount(void) { return 0; }
 Uint32 DebugDsp_OpcodeType(void) { return 0; }
 #endif
@@ -157,7 +151,7 @@ Uint32 DSP_ReadMemory(Uint16 addr, char space, const char **mem_str)
 
 /* fake console redirection */
 #include "console.h"
-int ConOutDevice;
+int ConOutDevices;
 void Console_Check(void) { }
 
 /* fake profiler stuff */
